# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Benchmarks for Atheris execs/sec performance."""
# In Google, must be built with --config=asan-fuzzer.

import atheris
import contextlib

# Use new atheris instrumentation only if on new atheris
if "instrument_func" in dir(atheris):
  instrument_func = atheris.instrument_func
  instrument_imports = atheris.instrument_imports
  instrument_all = atheris.instrument_all
else:

  def instrument_func(x):
    return x

  def instrument_all():
    pass

  @contextlib.contextmanager
  def instrument_imports(*args, **kwargs):
    yield None


with instrument_imports():
  import fcntl
  import os
  import sys
  import time
  import re
  import json
  import zipfile
  import io


def _set_nonblocking(fd):
  """Set the specified fd to a nonblocking mode."""
  oflags = fcntl.fcntl(fd, fcntl.F_GETFL)
  nflags = oflags | os.O_NONBLOCK
  fcntl.fcntl(fd, fcntl.F_SETFL, nflags)


def _benchmark_child(test_one_input, num_runs, pipe, args, inst_all):
  os.close(pipe[0])
  os.dup2(pipe[1], 1)
  os.dup2(pipe[1], 2)

  if inst_all:
    instrument_all()

  counter = [0]
  start = time.time()

  def wrapped_test_one_input(data):
    counter[0] += 1
    if counter[0] == num_runs:
      print(f"\nbenchmark_duration={time.time() - start}")
      os._exit(0)
    test_one_input(data)

  atheris.Setup([sys.argv[0]] + args, wrapped_test_one_input)
  atheris.Fuzz()
  assert False  # Does not return


def run_benchmark(test_one_input,
                  num_runs,
                  timeout=10,
                  inst_all=False,
                  args=[]):
  """Fuzz test_one_input() in a subprocess.

  This forks a child, and in the child, runs atheris.Setup(test_one_input) and
  atheris.Fuzz(). Expects the fuzzer to quickly find a crash.

  Args:
    test_one_input: a callable that takes a bytes.
    timeout: float. Time until the fuzzing is aborted and an assertion failure
      is raised.
    args: additional command-line arguments to pass to the fuzzing run.
  """
  pipe = os.pipe()

  pid = os.fork()
  if pid == 0:
    _benchmark_child(test_one_input, num_runs, pipe, args, inst_all)

  os.close(pipe[1])
  _set_nonblocking(pipe[0])

  stdout = b""
  start_time = time.time()
  while True:
    data = b""
    try:
      data = os.read(pipe[0], 1024)
    except BlockingIOError:
      pass

    stdout += data

    if len(data) != 0:
      continue

    wpid = os.waitpid(pid, os.WNOHANG)

    if wpid == (0, 0):
      # Process not done yet
      if time.time() > start_time + timeout:
        raise TimeoutError("Fuzz target failed to exit within expected time.")
      time.sleep(0.1)
      continue

    # Process done, get any remaining output.
    with os.fdopen(pipe[0], "rb") as f:
      data = f.read()
    stdout += data
    break

  result_line = re.search(b"benchmark_duration=[0-9.]+", stdout)
  time_taken = float(result_line[0].split(b"=")[1])
  runs_per_sec = num_runs / time_taken

  sys.stdout.write(
      f"{test_one_input.__name__}\truns={num_runs}\ttime={time_taken:.2f}s\texecs/sec={runs_per_sec:.2f}\n"
  )


@instrument_func
def low_cyclomatic(data):
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9
  x = 0
  x = 1
  x = 2
  x = 3
  x = 4
  x = 5
  x = 6
  x = 7
  x = 8
  x = 9


@instrument_func
def high_cyclomatic(data):
  for c in data:
    if c == 0:
      c = 38
    if c == 1:
      c = 201
    if c == 2:
      c = 192
    if c == 3:
      c = 70
    if c == 4:
      c = 184
    if c == 5:
      c = 100
    if c == 6:
      c = 85
    if c == 7:
      c = 0
    if c == 8:
      c = 18
    if c == 9:
      c = 183
    if c == 10:
      c = 140
    if c == 11:
      c = 216
    if c == 12:
      c = 60
    if c == 13:
      c = 139
    if c == 14:
      c = 133
    if c == 15:
      c = 252
    if c == 16:
      c = 148
    if c == 17:
      c = 156
    if c == 18:
      c = 73
    if c == 19:
      c = 137
    if c == 20:
      c = 167
    if c == 21:
      c = 44
    if c == 22:
      c = 90
    if c == 23:
      c = 50
    if c == 24:
      c = 169
    if c == 25:
      c = 216
    if c == 26:
      c = 182
    if c == 27:
      c = 231
    if c == 28:
      c = 192
    if c == 29:
      c = 14
    if c == 30:
      c = 236
    if c == 31:
      c = 158
    if c == 32:
      c = 38
    if c == 33:
      c = 36
    if c == 34:
      c = 101
    if c == 35:
      c = 75
    if c == 36:
      c = 81
    if c == 37:
      c = 105
    if c == 38:
      c = 217
    if c == 39:
      c = 33
    if c == 40:
      c = 200
    if c == 41:
      c = 124
    if c == 42:
      c = 161
    if c == 43:
      c = 81
    if c == 44:
      c = 6
    if c == 45:
      c = 231
    if c == 46:
      c = 156
    if c == 47:
      c = 213
    if c == 48:
      c = 203
    if c == 49:
      c = 121
    if c == 50:
      c = 217
    if c == 51:
      c = 170
    if c == 52:
      c = 217
    if c == 53:
      c = 249
    if c == 54:
      c = 201
    if c == 55:
      c = 81
    if c == 56:
      c = 205
    if c == 57:
      c = 206
    if c == 58:
      c = 50
    if c == 59:
      c = 131
    if c == 60:
      c = 223
    if c == 61:
      c = 24
    if c == 62:
      c = 220
    if c == 63:
      c = 83
    if c == 64:
      c = 15
    if c == 65:
      c = 186
    if c == 66:
      c = 126
    if c == 67:
      c = 68
    if c == 68:
      c = 94
    if c == 69:
      c = 101
    if c == 70:
      c = 85
    if c == 71:
      c = 229
    if c == 72:
      c = 10
    if c == 73:
      c = 7
    if c == 74:
      c = 57
    if c == 75:
      c = 124
    if c == 76:
      c = 111
    if c == 77:
      c = 230
    if c == 78:
      c = 192
    if c == 79:
      c = 111
    if c == 80:
      c = 237
    if c == 81:
      c = 106
    if c == 82:
      c = 126
    if c == 83:
      c = 149
    if c == 84:
      c = 28
    if c == 85:
      c = 204
    if c == 86:
      c = 241
    if c == 87:
      c = 113
    if c == 88:
      c = 161
    if c == 89:
      c = 136
    if c == 90:
      c = 189
    if c == 91:
      c = 156
    if c == 92:
      c = 195
    if c == 93:
      c = 123
    if c == 94:
      c = 64
    if c == 95:
      c = 93
    if c == 96:
      c = 45
    if c == 97:
      c = 167
    if c == 98:
      c = 218
    if c == 99:
      c = 211
    if c == 100:
      c = 19
    if c == 101:
      c = 94
    if c == 102:
      c = 207
    if c == 103:
      c = 128
    if c == 104:
      c = 209
    if c == 105:
      c = 13
    if c == 106:
      c = 71
    if c == 107:
      c = 39
    if c == 108:
      c = 218
    if c == 109:
      c = 124
    if c == 110:
      c = 51
    if c == 111:
      c = 204
    if c == 112:
      c = 94
    if c == 113:
      c = 171
    if c == 114:
      c = 1
    if c == 115:
      c = 190
    if c == 116:
      c = 1
    if c == 117:
      c = 248
    if c == 118:
      c = 216
    if c == 119:
      c = 10
    if c == 120:
      c = 162
    if c == 121:
      c = 204
    if c == 122:
      c = 152
    if c == 123:
      c = 196
    if c == 124:
      c = 146
    if c == 125:
      c = 227
    if c == 126:
      c = 191
    if c == 127:
      c = 182
    if c == 128:
      c = 100
    if c == 129:
      c = 74
    if c == 130:
      c = 82
    if c == 131:
      c = 129
    if c == 132:
      c = 207
    if c == 133:
      c = 51
    if c == 134:
      c = 168
    if c == 135:
      c = 81
    if c == 136:
      c = 101
    if c == 137:
      c = 108
    if c == 138:
      c = 30
    if c == 139:
      c = 66
    if c == 140:
      c = 21
    if c == 141:
      c = 135
    if c == 142:
      c = 248
    if c == 143:
      c = 49
    if c == 144:
      c = 203
    if c == 145:
      c = 20
    if c == 146:
      c = 135
    if c == 147:
      c = 197
    if c == 148:
      c = 212
    if c == 149:
      c = 159
    if c == 150:
      c = 173
    if c == 151:
      c = 195
    if c == 152:
      c = 152
    if c == 153:
      c = 158
    if c == 154:
      c = 27
    if c == 155:
      c = 61
    if c == 156:
      c = 209
    if c == 157:
      c = 155
    if c == 158:
      c = 55
    if c == 159:
      c = 87
    if c == 160:
      c = 229
    if c == 161:
      c = 143
    if c == 162:
      c = 200
    if c == 163:
      c = 220
    if c == 164:
      c = 164
    if c == 165:
      c = 97
    if c == 166:
      c = 92
    if c == 167:
      c = 65
    if c == 168:
      c = 253
    if c == 169:
      c = 249
    if c == 170:
      c = 23
    if c == 171:
      c = 1
    if c == 172:
      c = 154
    if c == 173:
      c = 248
    if c == 174:
      c = 89
    if c == 175:
      c = 144
    if c == 176:
      c = 109
    if c == 177:
      c = 233
    if c == 178:
      c = 46
    if c == 179:
      c = 174
    if c == 180:
      c = 101
    if c == 181:
      c = 130
    if c == 182:
      c = 116
    if c == 183:
      c = 103
    if c == 184:
      c = 142
    if c == 185:
      c = 229
    if c == 186:
      c = 217
    if c == 187:
      c = 207
    if c == 188:
      c = 146
    if c == 189:
      c = 24
    if c == 190:
      c = 153
    if c == 191:
      c = 149
    if c == 192:
      c = 48
    if c == 193:
      c = 77
    if c == 194:
      c = 187
    if c == 195:
      c = 205
    if c == 196:
      c = 75
    if c == 197:
      c = 171
    if c == 198:
      c = 122
    if c == 199:
      c = 37
    if c == 200:
      c = 92
    if c == 201:
      c = 63
    if c == 202:
      c = 71
    if c == 203:
      c = 81
    if c == 204:
      c = 101
    if c == 205:
      c = 216
    if c == 206:
      c = 32
    if c == 207:
      c = 55
    if c == 208:
      c = 219
    if c == 209:
      c = 204
    if c == 210:
      c = 63
    if c == 211:
      c = 16
    if c == 212:
      c = 6
    if c == 213:
      c = 49
    if c == 214:
      c = 158
    if c == 215:
      c = 228
    if c == 216:
      c = 237
    if c == 217:
      c = 123
    if c == 218:
      c = 161
    if c == 219:
      c = 15
    if c == 220:
      c = 172
    if c == 221:
      c = 79
    if c == 222:
      c = 64
    if c == 223:
      c = 34
    if c == 224:
      c = 147
    if c == 225:
      c = 228
    if c == 226:
      c = 200
    if c == 227:
      c = 0
    if c == 228:
      c = 12
    if c == 229:
      c = 209
    if c == 230:
      c = 95
    if c == 231:
      c = 239
    if c == 232:
      c = 161
    if c == 233:
      c = 24
    if c == 234:
      c = 211
    if c == 235:
      c = 35
    if c == 236:
      c = 187
    if c == 237:
      c = 79
    if c == 238:
      c = 116
    if c == 239:
      c = 46
    if c == 240:
      c = 184
    if c == 241:
      c = 170
    if c == 242:
      c = 199
    if c == 243:
      c = 118
    if c == 244:
      c = 237
    if c == 245:
      c = 174
    if c == 246:
      c = 142
    if c == 247:
      c = 99
    if c == 248:
      c = 137
    if c == 249:
      c = 163
    if c == 250:
      c = 28
    if c == 251:
      c = 233
    if c == 252:
      c = 19
    if c == 253:
      c = 189
    if c == 254:
      c = 143
    if c == 255:
      c = 7


def json_fuzz(data):
  try:
    json.loads(data.decode("utf-8", "surrogatepass"))
  except Exception as e:
    pass


@instrument_func
def zip_fuzz(data):
  try:
    with io.BytesIO(data) as f:
      pz = zipfile.ZipFile(f)
      pinfos = pz.infolist()

      for i in range(len(pinfos)):
        pinfo = pinfos[i]

        x = pinfo.comment
        x = pinfo.compress_size
        x = pinfo.external_attr
        x = pinfo.extra
        x = pinfo.filename
        x = pinfo.flag_bits
        x = pinfo.header_offset
        x = pinfo.internal_attr
        x = pinfo.is_dir()
        x = pinfo.reserved
        x = pinfo.volume

        with pz.open(pinfo) as g:
          pdata = g.read()
  except Exception as e:
    pass


run_benchmark(low_cyclomatic, num_runs=40000, timeout=30)
run_benchmark(high_cyclomatic, num_runs=2000, timeout=30)
run_benchmark(json_fuzz, num_runs=4000, timeout=120, inst_all=True)
run_benchmark(zip_fuzz, num_runs=4000, timeout=30)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys
import time
import unittest
import zlib
import functools

import atheris

import fuzz_test_lib


def fail_immediately(data):
  raise RuntimeError("Failed immediately")


@atheris.instrument_func
def many_branches(data):
  if len(data) < 4:
    return
  if data[0] != 12:
    return
  if data[1] != 5:
    return
  if data[2] != 0:
    return
  if data[3] != 123:
    return

  raise RuntimeError("Many branches")


@atheris.instrument_func
def never_fail(data):
  for d in data:
    if d == 0:
      pass
    elif d == 1:
      pass
    elif d == 2:
      pass


@atheris.instrument_func
def bytes_comparison(data):
  if data == b"foobarbazbiz":
    raise RuntimeError("Was foobarbazbiz")


@atheris.instrument_func
def string_comparison(data):
  try:
    if data.decode("utf-8") == "foobarbazbiz":
      raise RuntimeError("Was foobarbazbiz")
  except UnicodeDecodeError:
    pass


@atheris.instrument_func
def utf8_comparison(data):
  try:
    decoded = data.decode("utf-8")
    if decoded == "⾐∾ⶑ➠":
      raise RuntimeError(f"Was random unicode '{decoded}'")
  except UnicodeDecodeError:
    pass


@atheris.instrument_func
def timeout_py(data):
  del data
  time.sleep(100000000)


@atheris.instrument_func
def regex_match(data):
  if re.search(b"(Sun|Mon)day", data) is not None:
    raise RuntimeError("Was RegEx Match")


@atheris.instrument_func
def compressed_data(data):
  try:
    decompressed = zlib.decompress(data)
  except zlib.error:
    return

  if len(decompressed) < 2:
    return

  try:
    if decompressed.decode() == "FU":
      raise RuntimeError("Boom")
  except UnicodeDecodeError:
    pass


@atheris.instrument_func
def reserve_counter_after_fuzz_start(data):
  del data
  atheris._reserve_counter()


@functools.lru_cache(maxsize=None)
def instrument_once(func):
  """Instruments func, and verifies that this is the first time."""
  assert("__ATHERIS_INSTRUMENTED__" not in func.__code__.co_consts)
  atheris.instrument_func(func)
  assert("__ATHERIS_INSTRUMENTED__" in func.__code__.co_consts)


def foo(data):
  if data == b"foobar":
    raise RuntimeError("Code instrumented at runtime.")


def runtime_instrument_code(data):
  instrument_once(foo)
  foo(data)



class IntegrationTests(unittest.TestCase):

  def testFails(self):
    fuzz_test_lib.run_fuzztest(
        fail_immediately, expected_output=b"Failed immediately")

  def testManyBranches(self):
    fuzz_test_lib.run_fuzztest(
        many_branches, expected_output=b"Many branches", timeout=90)

  def testBytesComparison(self):
    fuzz_test_lib.run_fuzztest(
        bytes_comparison, expected_output=b"Was foobarbazbiz", timeout=30)

  def testStringComparison(self):
    fuzz_test_lib.run_fuzztest(
        string_comparison, expected_output=b"Was foobarbazbiz", timeout=30)

  def testUtf8Comparison(self):
    fuzz_test_lib.run_fuzztest(
        utf8_comparison, expected_output=b"Was random unicode", timeout=60)

  def testTimeoutPy(self):
    """This test verifies that timeout messages are recorded from -timeout."""
    fuzz_test_lib.run_fuzztest(
        timeout_py,
        args=["-timeout=1"],
        expected_output=b"most recent call first")
    fuzz_test_lib.run_fuzztest(
        timeout_py,
        args=["-timeout=1"],
        expected_output=b"ERROR: libFuzzer: timeout after")

  def testRegExMatch(self):
    fuzz_test_lib.run_fuzztest(
        regex_match,
        expected_output=b"Was RegEx Match",
        enabled_hooks=["RegEx"])

  def testExitsGracefullyOnPyFail(self):
    fuzz_test_lib.run_fuzztest(
        fail_immediately, expected_output=b"Exiting gracefully.")

  def testExitsGracefullyOnRunsOut(self):
    fuzz_test_lib.run_fuzztest(
        never_fail,
        args=["-atheris_runs=2"],
        expected_output=b"Exiting gracefully.")

  def testRunsOutCount(self):
    fuzz_test_lib.run_fuzztest(
        never_fail, args=["-atheris_runs=3"], expected_output=b"Done 3 in ")

  def testCompressedDataWithoutCustomMutator(self):
    try:
      fuzz_test_lib.run_fuzztest(compressed_data)
    except TimeoutError:  # Expected to timeout without a custom mutator.
      pass

  def testReserveCounterAfterFuzzStart(self):
    fuzz_test_lib.run_fuzztest(
        reserve_counter_after_fuzz_start,
        args=["-atheris_runs=2"],
        expected_output=b"Exiting gracefully.")

  def testInstrumentCodeWhileFuzzing(self):
    fuzz_test_lib.run_fuzztest(
        runtime_instrument_code,
        timeout=90,
        expected_output=b"Code instrumented at runtime.")


if __name__ == "__main__":
  unittest.main()



# Copyright 2021 Google Inc.
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Atheris is a coverage-guided Python fuzzing engine."""

from typing import List
from .function_hooks import enabled_hooks
from .function_hooks import gen_match
from .import_hook import instrument_imports
from .instrument_bytecode import instrument_all
from .instrument_bytecode import instrument_func
from .instrument_bytecode import patch_code
from .utils import path
# MyPy cannot find native code.
from .native import _reserve_counter  # type: ignore[import]
from .native import _trace_branch  # type: ignore[import]
from .native import _trace_cmp  # type: ignore[import]
from .native import _trace_regex_match  # type: ignore[import]
from .native import ALL_REMAINING  # type: ignore[import]
from .native import Fuzz  # type: ignore[import]
from .native import FuzzedDataProvider  # type: ignore[import]
from .native import Mutate  # type: ignore[import]
from .native import Setup  # type: ignore[import]

# PyInstaller Support
# PyInstaller doesn't automatically support lazy imports, which happens because
# we dynamically decide whether to import the with/without_libfuzzer versions of
# the core module. This function tells it where to look for a hook-atheris.py
# file.


def get_hook_dirs() -> List[str]:
  import os  # pylint: disable=g-import-not-at-top
  return [os.path.dirname(__file__)]



#!/usr/bin/python3

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example of fuzzing with a custom mutator in Python.

This is a Python translation of the example at:
https://github.com/google/fuzzing/blob/master/docs/structure-aware-fuzzing.md#example-compression.
"""

import atheris

with atheris.instrument_imports():
  import sys
  import zlib


def CustomMutator(data, max_size, seed):
  try:
    decompressed = zlib.decompress(data)
  except zlib.error:
    decompressed = b'Hi'
  else:
    decompressed = atheris.Mutate(decompressed, len(decompressed))
  return zlib.compress(decompressed)


@atheris.instrument_func  # Instrument the TestOneInput function itself
def TestOneInput(data):
  """The entry point for our fuzzer.

  This is a callback that will be repeatedly invoked with different arguments
  after Fuzz() is called.
  We translate the arbitrary byte string into a format our function being fuzzed
  can understand, then call it.

  Args:
    data: Bytestring coming from the fuzzing engine.
  """

  try:
    decompressed = zlib.decompress(data)
  except zlib.error:
    return

  if len(decompressed) < 2:
    return

  try:
    if decompressed.decode() == 'FU':
      raise RuntimeError('Boom')
  except UnicodeDecodeError:
    pass


if __name__ == '__main__':
  if len(sys.argv) > 1 and sys.argv[1] == '--no_mutator':
    atheris.Setup(sys.argv, TestOneInput)
  else:
    atheris.Setup(sys.argv, TestOneInput, custom_mutator=CustomMutator)
  atheris.Fuzz()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for FuzzedDataProvider pybind."""

import math
import random
import sys

import atheris

import unittest

if sys.version_info[0] >= 3:
  codepoint = chr

  def to_bytes(n, length):
    return n.to_bytes(length, "little")

else:
  # functionality from python3's chr() function is called unichr() in python2
  codepoint = unichr

  # functionality from python3's int.to_bytes()
  def to_bytes(n, length):
    h = "%x" % n
    s = ("0" * (len(h) % 2) + h).zfill(length * 2).decode("hex")
    return s[::-1]


ASCII_BYTEMARK = to_bytes(1, length=1)
UTF16_BYTEMARK = to_bytes(2, length=1)
UTF32_BYTEMARK = to_bytes(0, length=1)

HIGH_SURROGATE = to_bytes(0xDAA1, length=2)
LOW_SURROGATE = to_bytes(0xDD03, length=2)
BYTE_ORDER_BIG_ENDIAN = to_bytes(0xFEFF, length=2)
BYTE_ORDER_LITTLE_ENDIAN = to_bytes(0xFEFF, length=2)


class FuzzedDataProviderTest(unittest.TestCase):

  def testUnicodeActuallyAscii(self):
    fdp = atheris.FuzzedDataProvider(ASCII_BYTEMARK + b"abc123\0\x7f" +
                                     BYTE_ORDER_LITTLE_ENDIAN + b"\xd1")
    expected = "abc123\0\x7f\x7f\x7e\x51"
    actual = fdp.ConsumeUnicode(atheris.ALL_REMAINING)
    self.assertEqual(expected, actual)

  def testUnicode16(self):
    fdp = atheris.FuzzedDataProvider(UTF16_BYTEMARK + b"abc123\0\x7f" +
                                     HIGH_SURROGATE + LOW_SURROGATE + b"\xd1")

    expected = str()
    expected += b"abc123\0\x7f".decode("utf-16")
    expected += codepoint(0xDAA1) + codepoint(0xDD03)

    actual = fdp.ConsumeUnicode(atheris.ALL_REMAINING)
    self.assertEqual(expected, actual)

  def testUnicode16NoSurrogate(self):
    fdp = atheris.FuzzedDataProvider(UTF16_BYTEMARK + b"abc123\0\x7f" +
                                     HIGH_SURROGATE + LOW_SURROGATE + b"\xd1")

    high_surrogate_sub = to_bytes(0xDAA1 - 0xd800, length=2)
    low_surrogate_sub = to_bytes(0xDD03 - 0xd800, length=2)

    expected = str()
    expected += (b"abc123\0\x7f" + high_surrogate_sub +
                 low_surrogate_sub).decode("utf-16")

    actual = fdp.ConsumeUnicodeNoSurrogates(atheris.ALL_REMAINING)
    self.assertEqual(expected, actual)

  def testUnicode32(self):
    fdp = atheris.FuzzedDataProvider(UTF32_BYTEMARK + b"dc\x0e\0" + b"4321" +
                                     HIGH_SURROGATE + b"\0\0" + LOW_SURROGATE +
                                     b"\0\0" + b"\xd1")

    chunk1 = b"dc\x0e\0".decode("utf-32")
    chunk2 = codepoint(0x31323334 & 0x10ffff)
    chunk3 = codepoint(0xDAA1)
    chunk4 = codepoint(0xDD03)

    expected = (chunk1 + chunk2 + chunk3 + chunk4)

    actual = fdp.ConsumeUnicode(atheris.ALL_REMAINING)
    for i in range(len(actual)):
      self.assertEqual(ord(expected[i]), ord(actual[i]))
    self.assertEqual(len(expected), len(actual))
    self.assertEqual(expected, actual)

  def testUnicode32NoSurrogate(self):
    fdp = atheris.FuzzedDataProvider(UTF32_BYTEMARK + b"dc\x0e\0" + b"4321" +
                                     HIGH_SURROGATE + b"\0\0" + LOW_SURROGATE +
                                     b"\0\0" + b"\xd1")

    chunk1 = b"dc\x0e\0".decode("utf-32")
    chunk2 = codepoint(0x31323334 & 0x10ffff)
    chunk3 = codepoint(0xDAA1 - 0xd800)
    chunk4 = codepoint(0xDD03 - 0xd800)

    expected = (chunk1 + chunk2 + chunk3 + chunk4)

    actual = fdp.ConsumeUnicodeNoSurrogates(atheris.ALL_REMAINING)
    for i in range(len(actual)):
      self.assertEqual(ord(expected[i]), ord(actual[i]))
    self.assertEqual(len(expected), len(actual))
    self.assertEqual(expected, actual)

  def testBytes(self):
    expected = b"abc123\0\0xff\0x7f\0x80"
    fdp = atheris.FuzzedDataProvider(expected)

    self.assertEqual(expected, fdp.ConsumeBytes(atheris.ALL_REMAINING))

  def testString(self):
    if sys.version_info[0] >= 3:
      fdp = atheris.FuzzedDataProvider(ASCII_BYTEMARK + b"abc" +
                                       ASCII_BYTEMARK + b"123")
      self.assertEqual("abc", fdp.ConsumeString(3))
      self.assertEqual("123", fdp.ConsumeString(atheris.ALL_REMAINING))
    else:
      fdp = atheris.FuzzedDataProvider(b"abc123")
      self.assertEqual("abc", fdp.ConsumeString(3))
      self.assertEqual("123", fdp.ConsumeString(atheris.ALL_REMAINING))

  def testInt1(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(1), 0x01)

  def testUInt1(self):
    fdp = atheris.FuzzedDataProvider(b"\xe1\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeUInt(1), 0xe1)

  def testNegInt1(self):
    fdp = atheris.FuzzedDataProvider(b"\x81\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(1), 0x81 - 0x100)

  def testInt2(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(2), 0x0201)

  def testUInt2(self):
    fdp = atheris.FuzzedDataProvider(b"\xa1\xe2\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeUInt(2), 0xe2a1)

  def testNegInt2(self):
    fdp = atheris.FuzzedDataProvider(b"\x10\x82\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(2), 0x8210 - 0x10000)

  def testInt3(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(3), 0x030201)

  def testUInt3(self):
    fdp = atheris.FuzzedDataProvider(b"\xa1\xb2\xc3\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeUInt(3), 0xc3b2a1)

  def testNegInt3(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\xd3\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(3), 0xd30201 - 0x1000000)

  def testInt4(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(4), 0x4030201)

  def testUInt4(self):
    fdp = atheris.FuzzedDataProvider(b"\xa1\xb2\xc3\xd4\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeUInt(4), 0xd4c3b2a1)

  def testNegInt4(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\xe4\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(4), 0xe4030201 - 0x100000000)

  def testInt8(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\x08")
    self.assertEqual(fdp.ConsumeInt(8), 0x0807060504030201)

  def testNegInt8(self):
    fdp = atheris.FuzzedDataProvider(b"\x01\x02\x03\x04\x05\x06\x07\xb8")
    self.assertEqual(
        fdp.ConsumeInt(8), 0xb807060504030201 - 0x10000000000000000)

  def testInt9(self):
    fdp = atheris.FuzzedDataProvider(
        b"\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a")
    self.assertEqual(fdp.ConsumeInt(9), 0x090807060504030201)

  def testNegInt9(self):
    fdp = atheris.FuzzedDataProvider(
        b"\x01\x02\x03\x04\x05\x06\x07\x08\xe9\x0a")
    self.assertEqual(
        fdp.ConsumeInt(9), 0xe90807060504030201 - 0x1000000000000000000)

  def testUInt9(self):
    fdp = atheris.FuzzedDataProvider(
        b"\xa1\xb2\xc3\xd4\xe5\xf6\xa7\xb8\xc9\xda")
    self.assertEqual(fdp.ConsumeUInt(9), 0xc9b8a7f6e5d4c3b2a1)

  def testInRange1(self):
    arr = b""
    for _ in range(0, 1000):
      arr += to_bytes(random.randint(0, 255), 1)
    fdp = atheris.FuzzedDataProvider(arr)

    for _ in range(0, 1000):
      one = random.randint(0, 255)
      two = random.randint(0, 255)
      if two >= one:
        result = fdp.ConsumeIntInRange(one, two)
        self.assertGreaterEqual(result, one)
        self.assertLessEqual(result, two)
      else:
        result = fdp.ConsumeIntInRange(two, one)
        self.assertGreaterEqual(result, two)
        self.assertLessEqual(result, one)

  def testInRange9(self):
    arr = b""
    for _ in range(0, 1000):
      arr += to_bytes(random.randint(0, 255), 1)
    fdp = atheris.FuzzedDataProvider(arr)

    for _ in range(0, 1000):
      one = random.randint(2**64, 2**72)
      two = random.randint(2**64, 2**72)
      if two >= one:
        result = fdp.ConsumeIntInRange(one, two)
        self.assertGreaterEqual(result, one)
        self.assertLessEqual(result, two)
      else:
        result = fdp.ConsumeIntInRange(two, one)
        self.assertGreaterEqual(result, two)
        self.assertLessEqual(result, one)

  def testIntList1(self):
    arr = b""
    for _ in range(0, 1000):
      arr += to_bytes(random.randint(0, 255), 1)
    fdp = atheris.FuzzedDataProvider(arr)

    l = fdp.ConsumeIntList(4321, 1)
    self.assertEqual(len(l), 4321)
    for i in range(0, 1000):
      if arr[i] < 0:
        arr[i] += 256
      if l[i] < 0:
        l[i] += 256
      if sys.version_info[0] >= 3:
        self.assertEqual(arr[i], l[i])
      else:
        self.assertEqual(ord(arr[i]), l[i])

    for i in range(1000, 4321):
      self.assertEqual(l[i], 0)

  def testIntList9(self):
    arr = b""
    for _ in range(0, 1000):
      arr += to_bytes(random.randint(0, 2**72 - 1), 9)
    fdp = atheris.FuzzedDataProvider(arr)

    l = fdp.ConsumeIntList(4321, 9)
    self.assertEqual(len(l), 4321)

    for i in range(0, 1000):
      self.assertGreaterEqual(l[i], -2**71)
      self.assertLessEqual(l[i], 2**71 - 1)

    for i in range(1000, 4321):
      self.assertEqual(l[i], 0)

  def testNonInfiniteFloat(self):
    arr = []
    arr.append(50)
    arr.append(152)
    arr.append(217)
    arr.append(85)
    arr.append(209)
    arr.append(188)
    arr.append(146)
    arr.append(14)
    arr.append(201)
    arr.append(240)
    arr = b"".join([to_bytes(x, 1) for x in arr])

    fdp = atheris.FuzzedDataProvider(arr)
    val = fdp.ConsumeFloat()
    self.assertGreaterEqual(val, -1.7976931348623157e+308)
    self.assertLessEqual(val, 1.7976931348623157e+308)

  def testFloatList(self):
    arr = b""
    for i in range(0, 256):
      arr += to_bytes(i, 1)
      arr += to_bytes(random.getrandbits(72), 9)

    fdp = atheris.FuzzedDataProvider(arr)

    pos_zero = fdp.ConsumeFloat()
    self.assertEqual(pos_zero, 0.0)
    self.assertNotEqual(str(pos_zero)[0], "-")

    neg_zero = fdp.ConsumeFloat()
    self.assertEqual(neg_zero, 0.0)
    self.assertEqual(str(neg_zero)[0], "-")

    pos_inf = fdp.ConsumeFloat()
    self.assertTrue(math.isinf(pos_inf))
    self.assertGreater(pos_inf, 0.0)

    neg_inf = fdp.ConsumeFloat()
    self.assertTrue(math.isinf(neg_inf))
    self.assertLess(neg_inf, 0)

    nan = fdp.ConsumeFloat()
    self.assertTrue(math.isnan(nan))

    pos_denorm_min = fdp.ConsumeFloat()
    self.assertLess(pos_denorm_min, 1e-323)
    self.assertGreater(pos_denorm_min, 0)

    neg_denorm_min = fdp.ConsumeFloat()
    self.assertGreater(neg_denorm_min, -1e-323)
    self.assertLess(neg_denorm_min, 0)

    pos_regular_min = fdp.ConsumeFloat()
    self.assertLess(pos_regular_min, 2.3e-308)
    self.assertGreater(pos_regular_min, 2.1e-308)

    neg_regular_min = fdp.ConsumeFloat()
    self.assertGreater(neg_regular_min, -2.3e-308)
    self.assertLess(neg_regular_min, -2.1e-308)

    pos_max = fdp.ConsumeFloat()
    self.assertGreater(pos_max, 1.79769313e+308)

    neg_max = fdp.ConsumeFloat()
    self.assertLess(neg_max, -1.79769313e+308)

    for _ in range(11, 256):
      val = fdp.ConsumeFloat()
      self.assertGreater(val, -1.79769313e+308)
      self.assertLess(val, 1.79769313e+308)

  def testPickValueInList1(self):
    l = [3, 3]

    arr = to_bytes(random.getrandbits(1024), int(1024 / 8))
    arr = to_bytes(1234, 8) + arr

    fdp = atheris.FuzzedDataProvider(arr)
    for _ in range(0, int(1024 / 8)):
      self.assertEqual(fdp.PickValueInList(l), 3)
    self.assertEqual(fdp.ConsumeIntInRange(0, 2**64 - 1), 1234)

    self.assertEqual(fdp.PickValueInList(l), 3)

  def testPickValueInList7(self):
    l = [4, 17, 52, 12, 8, 71, 2]
    s = set()

    arr = to_bytes(random.getrandbits(1024 * 1024), int(1024 * 1024 / 8))
    fdp = atheris.FuzzedDataProvider(arr)

    for _ in range(0, int(1024 * 1024 / 8)):
      s.add(fdp.PickValueInList(l))

    self.assertEqual(s, set(l))
    self.assertEqual(fdp.PickValueInList(l), 4)

  def testPickValueInListShort(self):
    l = []
    for i in range(1, 10001):
      l.append(i * 13)

    arr = to_bytes(random.getrandbits(1024 * 1024), int(1024 * 1024 / 8))
    fdp = atheris.FuzzedDataProvider(arr)

    all_returned = set()
    for i in range(0, 10000):
      val = fdp.PickValueInList(l)
      self.assertEqual(val % 13, 0)
      all_returned.add(val)

    self.assertGreater(len(all_returned), 200)


if __name__ == "__main__":
  unittest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for RegEx match generation."""

import re
import sre_parse
import unittest
from atheris import gen_match


class RegexMatchGeneration(unittest.TestCase):

  def test_plain(self):
    match = gen_match(sre_parse.parse("abc"))
    self.assertEqual(match, "abc")

  def test_alternate1(self):
    match = gen_match(sre_parse.parse("abc|def"))
    self.assertIn(match, ["abc", "def"])

  def test_alternate2(self):
    pattern = r"(abc|\d+)"
    match = gen_match(sre_parse.parse(pattern))
    self.assertRegex(match, pattern)

  def test_oneof(self):
    match = gen_match(sre_parse.parse("[abc]abc"))
    self.assertIn(match, ["aabc", "babc", "cabc"])

  def test_notoneof(self):
    match = gen_match(sre_parse.parse("[^abc]def"))
    if len(match) != 4:
      raise AssertionError(f"Unexpected generated match {match}")
    if not match.endswith("def"):
      raise AssertionError(f"Unexpected generated match {match}")
    if match[0] in "abc":
      raise AssertionError(f"Unexpected generated match {match}")

  def test_noncapturing(self):
    pattern = r"(?:abc){3,}"
    match = gen_match(sre_parse.parse(pattern))
    self.assertRegex(match, pattern)
    print(match)

  def test_noncapturing2(self):
    pattern = r"(?:abc){,3}"
    match = gen_match(sre_parse.parse(pattern))
    self.assertRegex(match, pattern)
    print(match)

  def test_lookahead(self):
    pattern = r"y(?=abc)"
    match = gen_match(sre_parse.parse(pattern))
    self.assertRegex(match, pattern)

  def test_unicode(self):
    match = gen_match(sre_parse.parse("•"))
    self.assertEqual(match, "•")

  # Unsupported yet:
  # def test_plainbytes(self):
  #   match = gen_match(sre_parse.parse(b"abc"))
  #   self.assertEqual(match, b"abc")
  # def test_negative_lookbehind(self):
  #   pattern = r"t(?<!abc)u"
  #   match = gen_match(sre_parse.parse(pattern))
  #   self.assertRegex(match, pattern)
  # def test_digits(self):
  #   pattern = r"\d"
  #   match = gen_match(sre_parse.parse(pattern))
  #   self.assertRegex(match, pattern)




if __name__ == "__main__":
  unittest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
import sys
import setuptools

__version__ = '0.0.1'


class get_pybind_include(object):

  def __str__(self):
    import pybind11
    return pybind11.get_include()


ext_modules = [
    Extension(
        'libidn2',
        ['libidn2.cc'],
        include_dirs=[
            # Path to pybind11 headers
            get_pybind_include(),
        ],
        language='c++'),
]


class BuildExt(build_ext):
  """A custom build extension for adding compiler-specific options."""
  c_opts = {
      'unix': [],
  }
  l_opts = {
      'unix': ['-lidn2'],
  }

  if sys.platform == 'darwin':
    darwin_opts = ['-stdlib=libc++', '-mmacosx-version-min=10.7']
    c_opts['unix'] += darwin_opts
    l_opts['unix'] += darwin_opts

  def build_extensions(self):
    ct = self.compiler.compiler_type
    opts = self.c_opts.get(ct, [])
    link_opts = self.l_opts.get(ct, [])
    if ct == 'unix':
      opts.append('--std=c++11')

    for ext in self.extensions:
      ext.define_macros = [('VERSION_INFO',
                            '"{}"'.format(self.distribution.get_version()))]
      ext.extra_compile_args = opts
      ext.extra_link_args = link_opts
    build_ext.build_extensions(self)


setup(
    name='libidn2',
    version=__version__,
    author='Bitshift',
    author_email='atheris@google.com',
    url='',
    description='A simple wrapper around libidn2, for use with idna_fuzzer.py',
    long_description='',
    ext_modules=ext_modules,
    setup_requires=['pybind11>=2.5.0'],
    cmdclass={'build_ext': BuildExt},
    zip_safe=False,
)



# Copyright 2021 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Runs coverage_test with PyInstaller to ensure coverage still functions."""

import os
import subprocess
import sys
import unittest


class PyInstallerCoverageTest(unittest.TestCase):

  def test_build_and_run(self):
    coverage_test = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "coverage_test.py")
    subprocess.check_call([
        sys.executable, "-m", "PyInstaller", "--distpath", "/tmp/pyinstaller",
        "--onefile", "--name", "coverage_test.pkg", coverage_test
    ])
    subprocess.check_call(["/tmp/pyinstaller/coverage_test.pkg"])


if __name__ == "__main__":
  unittest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import unittest
import zlib

import atheris

import fuzz_test_lib


def compressed_mutator(data, max_size, seed):
  try:
    decompressed = zlib.decompress(data)
  except zlib.error:
    decompressed = b"Hi"
  else:
    decompressed = atheris.Mutate(decompressed, len(decompressed))
  return zlib.compress(decompressed)


@atheris.instrument_func
def compressed_data(data):
  try:
    decompressed = zlib.decompress(data)
  except zlib.error:
    return

  if len(decompressed) < 2:
    return

  try:
    if decompressed.decode() == "FU":
      raise RuntimeError("Boom")
  except UnicodeDecodeError:
    pass


class CustomMutatorTests(unittest.TestCase):

  def testCompressedData(self):
    fuzz_test_lib.run_fuzztest(
        compressed_data,
        setup_kwargs={"custom_mutator": compressed_mutator},
        expected_output=b"Boom")


if __name__ == "__main__":
  unittest.main()



#!/usr/bin/python3

# Copyright 2020 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This fuzzer is an example of looking for weird exceptions in ruamel.yaml.

While these do not necessarily indicate vulnerabilities, they often indicate
other bugs in code.
Two bugs found by this fuzzer include situations where ruamel.yaml tries
inserting lists/dicts as *keys* in maps, which is invalid JSON and causes a
ValueError to be raised. The worst consequences of this is likely
denial-of-service for code that only expects a YAMLError to be raised; however,
it may indicate more interesting parsing problems.
"""

import atheris

with atheris.instrument_imports():
  from ruamel import yaml as ruamel_yaml
  import sys
  import warnings

# Suppress all warnings.
warnings.simplefilter("ignore")

ryaml = ruamel_yaml.YAML(typ="safe", pure=True)
ryaml.allow_duplicate_keys = True


@atheris.instrument_func
def TestOneInput(input_bytes):
  fdp = atheris.FuzzedDataProvider(input_bytes)
  data = fdp.ConsumeUnicode(sys.maxsize)

  try:
    iterator = ryaml.load_all(data)
    for _ in iterator:
      pass
  except ruamel_yaml.error.YAMLError:
    return

  except Exception:
    input_type = str(type(data))
    codepoints = [hex(ord(x)) for x in data]
    sys.stderr.write(
        "Input was {input_type}: {data}\nCodepoints: {codepoints}".format(
            input_type=input_type, data=data, codepoints=codepoints))
    raise


def main():
  atheris.Setup(sys.argv, TestOneInput)
  atheris.Fuzz()


if __name__ == "__main__":
  main()



# Copyright 2021 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os


def path():
  dir, _ = os.path.split(sys.modules["atheris"].__file__)
  dir, _ = os.path.split(dir)
  return dir


class ProgressRenderer:
  """Displays an updating progress meter in the terminal."""

  def __init__(self, stream, total_count):
    assert stream.isatty()
    self.stream = stream

    self._count = 0
    self.total_count = total_count

    self._current_width = 0
    self.render()

  def render(self):
    self.erase()
    done_percent = int(100 * self._count / self.total_count)
    message = f"[{self._count}/{self.total_count}] {done_percent}%"
    self.stream.write(message)
    self.stream.flush()
    self._current_width = len(message)

  def erase(self):
    self.stream.write(("\b" * self._current_width) +
                      (" " * self._current_width) +
                      ("\b" * self._current_width))
    self.stream.flush()
    self._current_width = 0

  def drop(self):
    self._current_width = 0
    sys.stderr.write("\n")

  @property
  def count(self):
    return self._count

  @count.setter
  def count(self, new_count):
    self._count = new_count
    self.render()



# Copyright 2021 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""atheris instruments modules at import-time.

The instrument() function temporarily installs an import hook
(AtherisMetaPathFinder) in sys.meta_path that employs a custom loader
(AtherisSourceFileLoader, AtherisSourcelessFileLoader).
"""
# _frozen_importlib is a special Py Interpreter library, disable import-error.
import _frozen_importlib  # type: ignore[import]
import _frozen_importlib_external  # type: ignore[import]
from importlib import abc
from importlib import machinery
import sys
import types
from typing import Set, Optional, Sequence, Type, Union, Any
from .instrument_bytecode import patch_code

_warned_experimental = False

# A list of known loaders we should silence warnings about.
SKIP_LOADERS = set([
    # Google3 loader, implemented in native code, loads other native code.
    "StaticMetaImporter",
    # Google3 loader, implemented in native code, loads other native code as
    # well as Python code.
    "ElfZipImporter",
])


# TODO(b/207008147) Mypy does not like abc.FileLoader?
def _should_skip(loader: Any) -> bool:
  """Returns whether modules loaded with this importer should be ignored."""
  if hasattr(loader, "__qualname__"):
    if loader.__qualname__ in SKIP_LOADERS:  # type: ignore[attr-defined]
      return True

  if hasattr(loader.__class__, "__qualname__"):
    if loader.__class__.__qualname__ in SKIP_LOADERS:
      return True

  return False


class AtherisMetaPathFinder(abc.MetaPathFinder):
  """Finds and loads package metapaths with Atheris loaders."""

  def __init__(self, include_packages: Set[str], exclude_modules: Set[str],
               enable_loader_override: bool, trace_dataflow: bool):
    """Finds and loads package metapaths with Atheris loaders.

    Args:
      include_packages: If not empty, an allowlist of packages to instrument.
      exclude_modules: A denylist of modules to never instrument. This has
        higher precedent than include_packages.
      enable_loader_override: Use experimental support to instrument bytecode
        loaded from custom loaders.
      trace_dataflow: Whether or not to trace dataflow.
    """
    super().__init__()
    self._include_packages = include_packages
    self._exclude_modules = exclude_modules
    self._trace_dataflow = trace_dataflow
    self._enable_loader_override = enable_loader_override

  def find_spec(
      self,
      fullname: str,
      path: Optional[Sequence[Union[bytes, str]]],
      target: Optional[types.ModuleType] = None
  ) -> Optional[machinery.ModuleSpec]:
    """Returns the module spec if any.

    Args:
      fullname: Fully qualified name of the package.
      path: Parent package's __path__
      target: When passed in, target is a module object that the finder may use
        to make a more educated guess about what spec to return.

    Returns:
      The ModuleSpec if found, not excluded, and included if any are included.
    """
    if fullname in self._exclude_modules:
      return None

    package_name = fullname.split(".")[0]

    if (not self._include_packages or
        package_name in self._include_packages) and package_name != "atheris":
      # Try each importer after the Atheris importer until we find an acceptable
      # one
      found_atheris = False
      for meta in sys.meta_path:
        # Skip any loaders before (or including) the Atheris loader
        if not found_atheris:
          if meta is self:
            found_atheris = True
          continue

        # Check each remaining loader
        if not hasattr(meta, "find_spec"):
          continue

        spec = meta.find_spec(fullname, path, target)
        if spec is None or spec.loader is None:
          continue

        if _should_skip(spec.loader):
          return None

        if isinstance(spec.loader, machinery.ExtensionFileLoader):
          # An extension, coverage doesn't come from Python
          return None

        sys.stderr.write(f"INFO: Instrumenting {fullname}\n")

        # Use normal inheritance for the common cases. This may not be needed
        # (the dynamic case should work for everything), but keep this for as
        # long as that's experimental.
        if isinstance(spec.loader, _frozen_importlib_external.SourceFileLoader):
          spec.loader = AtherisSourceFileLoader(spec.loader.name,
                                                spec.loader.path,
                                                self._trace_dataflow)
          return spec

        elif isinstance(spec.loader,
                        _frozen_importlib_external.SourcelessFileLoader):
          spec.loader = AtherisSourcelessFileLoader(spec.loader.name,
                                                    spec.loader.path,
                                                    self._trace_dataflow)
          return spec

        else:
          # The common case isn't what we have, so wrap an existing object
          # via composition.

          if not self._enable_loader_override:
            sys.stderr.write("WARNING: Skipping import with custom loader.\n")
            return None

          global _warned_experimental
          if not _warned_experimental:
            sys.stderr.write(
                "WARNING: It looks like this module is imported by a "
                "custom loader. Atheris has experimental support for this. "
                "However, it may be incompatible with certain libraries. "
                "If you experience unusual errors or poor coverage "
                "collection, try atheris.instrument_all() instead, add "
                "enable_loader_override=False to instrument_imports(), or "
                "file an issue on GitHub.\n")
            _warned_experimental = True

          try:
            spec.loader = make_dynamic_atheris_loader(spec.loader,
                                                      self._trace_dataflow)
            return spec
          except Exception:  # pylint: disable=broad-except
            sys.stderr.write("WARNING: This module uses a custom loader that "
                             "prevents it from being instrumented: "
                             f"{spec.loader}\n")
            return None

          return None
      return None
    return None

  def invalidate_caches(self) -> None:
    return machinery.PathFinder.invalidate_caches()


class AtherisSourceFileLoader(_frozen_importlib_external.SourceFileLoader):
  """Loads a source file, patching its bytecode with Atheris instrumentation."""

  def __init__(self, name: str, path: str, trace_dataflow: bool):
    super().__init__(name, path)
    self._trace_dataflow = trace_dataflow

  def get_code(self, fullname: str) -> Optional[types.CodeType]:
    code = super().get_code(fullname)

    if code is None:
      return None
    else:
      return patch_code(code, self._trace_dataflow)


class AtherisSourcelessFileLoader(
    _frozen_importlib_external.SourcelessFileLoader):
  """Loads a sourceless/bytecode file, patching it with Atheris instrumentation."""

  def __init__(self, name: str, path: str, trace_dataflow: bool):
    super().__init__(name, path)
    self._trace_dataflow = trace_dataflow

  def get_code(self, fullname: str) -> Optional[types.CodeType]:
    code = super().get_code(fullname)

    if code is None:
      return None
    else:
      return patch_code(code, self._trace_dataflow)


def make_dynamic_atheris_loader(loader: Any, trace_dataflow: bool) -> Any:
  """Create a loader via 'object inheritance' and return it.

  This technique allows us to override just the get_code function on an
  already-existing object loader. This is experimental.

  Args:
    loader: Loader or Loader class.
    trace_dataflow: Whether or not to trace dataflow.

  Returns:
    The loader class overriden with Atheris tracing.
  """
  if loader.__class__ is type:
    # This is a class with classmethods. Use regular inheritance to override
    # get_code.

    class DynAtherisLoaderClass(loader):  # type: ignore[valid-type, misc]

      @classmethod
      def get_code(cls, fullname: str) -> Optional[types.CodeType]:
        code = loader.get_code(fullname)

        if code is None:
          return None
        return patch_code(code, cls._trace_dataflow)

    return DynAtherisLoaderClass

  # This is an object. We create a new object that's a copy of the existing
  # object but with a custom get_code implementation.
  class DynAtherisLoaderObject(loader.__class__):  # type: ignore[name-defined]
    """Dynamic wrapper over a loader."""

    def __init__(self, trace_dataflow: bool):
      self._trace_dataflow = trace_dataflow

    def get_code(self, fullname: str) -> Optional[types.CodeType]:
      code = super().get_code(fullname)

      if code is None:
        return None
      return patch_code(code, self._trace_dataflow)

  ret = DynAtherisLoaderObject(trace_dataflow)
  for k, v in loader.__dict__.items():
    if k not in ret.__dict__:
      ret.__dict__[k] = v

  return ret


class HookManager:
  """A Context manager that manages hooks."""

  def __init__(self, include_packages: Set[str], exclude_modules: Set[str],
               enable_loader_override: bool, trace_dataflow: bool):
    self._include_packages = include_packages
    self._exclude_modules = exclude_modules
    self._enable_loader_override = enable_loader_override
    self._trace_dataflow = trace_dataflow

  def __enter__(self) -> "HookManager":
    i = 0
    while i < len(sys.meta_path):
      if isinstance(sys.meta_path[i], AtherisMetaPathFinder):
        return self
      i += 1

    i = 0
    while i < len(sys.meta_path) and sys.meta_path[i] in [
        _frozen_importlib.BuiltinImporter, _frozen_importlib.FrozenImporter
    ]:
      i += 1

    sys.meta_path.insert(
        i,
        AtherisMetaPathFinder(self._include_packages, self._exclude_modules,
                              self._enable_loader_override,
                              self._trace_dataflow))

    return self

  def __exit__(self, *args: Any) -> None:
    i = 0
    while i < len(sys.meta_path):
      if isinstance(sys.meta_path[i], AtherisMetaPathFinder):
        sys.meta_path.pop(i)
      else:
        i += 1


def instrument_imports(include: Optional[Sequence[str]] = None,
                       exclude: Optional[Sequence[str]] = None,
                       enable_loader_override: bool = True) -> HookManager:
  """Returns a context manager that will instrument modules as imported.

  Args:
    include: module names that shall be instrumented. Submodules within these
      packages will be recursively instrumented too.
    exclude: module names that shall not be instrumented.
    enable_loader_override: Whether or not to enable the experimental feature of
      instrumenting custom loaders.

  Returns:

  Raises:
    TypeError: If any module name is not a str.
    ValueError: If any module name is a relative path or empty.
  """
  include = [] if include is None else list(include)
  exclude = [] if exclude is None else list(exclude)

  include_packages = set()

  for module_name in include + exclude:
    if not isinstance(module_name, str):
      raise TypeError("atheris.instrument_imports() expects names of " +
                      "modules of type <str>")
    elif not module_name:
      raise ValueError("atheris.instrument_imports(): " +
                       "You supplied an empty module name")
    elif module_name[0] == ".":
      raise ValueError("atheris.instrument_imports(): Please specify fully " +
                       "qualified module names (absolute not relative)")

  for module_name in include:
    if "." in module_name:
      module_name = module_name.split(".")[0]

    include_packages.add(module_name)

  return HookManager(
      include_packages,
      set(exclude),
      enable_loader_override,
      trace_dataflow=True)



#!/usr/bin/python3

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This fuzzer is an example of native extension fuzzing with coverage.

The ujson library should be built for coverage and,
optionally, Address Sanitizer.
(see build_install_ujson.sh and the instructions for sanitizers:
https://github.com/google/atheris/blob/master/using_sanitizers.md)
(see build_install_ujson.sh and the instructions for sanitizers:
https://github.com/google/atheris/blob/master/using_sanitizers.md)

This fuzzer is provided mainly as an example for how to deal with native
coverage.
"""

import sys
import atheris

# Here atheris.instrument_imports() is not necessary because ujson is just an
# extension, and we don't have any Python code we want to fuzz.
# Only python code is instrumented with atheris.instrument_imports();
# extensions are instrumented at compile-time.
import ujson


def TestOneInput(input_bytes):
  fdp = atheris.FuzzedDataProvider(input_bytes)
  original = fdp.ConsumeUnicode(sys.maxsize)

  try:
    ujson_data = ujson.loads(original)
  except ValueError:
    return

  # We make sure there's no error in encoding, but we don't actually compare
  # (encoded == original) because it's not entirely preserving. For example,
  # it does not preserve whitespace.
  encoded = ujson.dumps(ujson_data)
  del encoded


def main():
  atheris.Setup(sys.argv, TestOneInput)
  atheris.Fuzz()


if __name__ == "__main__":
  main()



# Copyright 2021 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module manages the version specific aspects of bytecode instrumentation.

Accross Python versions there are variations in:
    - Instructions
    - Instruction arguments
    - Shape of a code object
    - Construction of the lnotab

Currently supported python versions are:
    - 3.6
    - 3.7
    - 3.8
    - 3.9
    - 3.10
"""

import sys
import types

PYTHON_VERSION = sys.version_info[:2]

if PYTHON_VERSION < (3, 6) or PYTHON_VERSION > (3, 10):
  raise RuntimeError(
      "You are fuzzing on an unsupported python version: " +
      f"{PYTHON_VERSION[0]}.{PYTHON_VERSION[1]}. Only 3.6 - 3.10 are " +
      "supported by atheris 2.0. Use atheris 1.0 for older python versions."
  )

### Instruction categories ###

CONDITIONAL_JUMPS = [
    # common
    "FOR_ITER",
    "JUMP_IF_FALSE_OR_POP",
    "JUMP_IF_TRUE_OR_POP",
    "POP_JUMP_IF_FALSE",
    "POP_JUMP_IF_TRUE",

    # 3.9
    "JUMP_IF_NOT_EXC_MATCH",
]

UNCONDITIONAL_JUMPS = [
    # common
    "JUMP_FORWARD",
    "JUMP_ABSOLUTE",

    # 3.6 / 3.7
    "CONTINUE_LOOP",

    # 3.8
    "CALL_FINALLY",
]

ENDS_FUNCTION = [
    # common
    "RAISE_VARARGS",
    "RETURN_VALUE",

    # 3.9
    "RERAISE",
]

HAVE_REL_REFERENCE = [
    # common
    "SETUP_WITH",
    "JUMP_FORWARD",
    "FOR_ITER",
    "SETUP_FINALLY",
    "CALL_FINALLY",

    # 3.6 / 3.7
    "SETUP_LOOP",
    "SETUP_EXCEPT",
]

HAVE_ABS_REFERENCE = [
    # common
    "POP_JUMP_IF_TRUE",
    "POP_JUMP_IF_FALSE",
    "JUMP_IF_TRUE_OR_POP",
    "JUMP_IF_FALSE_OR_POP",
    "JUMP_ABSOLUTE",

    # 3.6 / 3.7
    "CONTINUE_LOOP",

    # 3.9
    "JUMP_IF_NOT_EXC_MATCH",
]

### Compare ops ###

REVERSE_CMP_OP = [4, 5, 2, 3, 0, 1]

### CodeTypes ###

if (3, 6) <= PYTHON_VERSION <= (3, 7):

  def get_code_object(code_obj, stacksize, bytecode, consts, names, lnotab):
    return types.CodeType(code_obj.co_argcount, code_obj.co_kwonlyargcount,
                          code_obj.co_nlocals, stacksize, code_obj.co_flags,
                          bytecode, consts, names, code_obj.co_varnames,
                          code_obj.co_filename, code_obj.co_name,
                          code_obj.co_firstlineno, lnotab, code_obj.co_freevars,
                          code_obj.co_cellvars)

else:

  def get_code_object(code_obj, stacksize, bytecode, consts, names, lnotab):
    return types.CodeType(code_obj.co_argcount, code_obj.co_posonlyargcount,
                          code_obj.co_kwonlyargcount, code_obj.co_nlocals,
                          stacksize, code_obj.co_flags, bytecode, consts, names,
                          code_obj.co_varnames, code_obj.co_filename,
                          code_obj.co_name, code_obj.co_firstlineno, lnotab,
                          code_obj.co_freevars, code_obj.co_cellvars)


### Python 3.10 uses instruction (2 byte) offsets rather than byte offsets ###

if PYTHON_VERSION >= (3, 10):

  def jump_arg_bytes(arg: int) -> int:
    return arg * 2

  def add_bytes_to_jump_arg(arg: int, size: int) -> int:
    return arg + size // 2
else:

  def jump_arg_bytes(arg: int) -> int:
    return arg

  def add_bytes_to_jump_arg(arg: int, size: int) -> int:
    return arg + size


### Lnotab handling ###

if (3, 6) <= PYTHON_VERSION <= (3, 10):

  def get_lnotab(code, listing):
    """Returns line number table."""
    lnotab = []
    current_lineno = listing[0].lineno
    i = 0

    assert listing[0].lineno >= code.co_firstlineno

    if listing[0].lineno > code.co_firstlineno:
      delta_lineno = listing[0].lineno - code.co_firstlineno

      while delta_lineno > 127:
        lnotab.extend([0, 127])
        delta_lineno -= 127

      lnotab.extend([0, delta_lineno])

    while True:
      delta_bc = 0

      while i < len(listing) and listing[i].lineno == current_lineno:
        delta_bc += listing[i].get_size()
        i += 1

      if i >= len(listing):
        break

      assert delta_bc > 0

      delta_lineno = listing[i].lineno - current_lineno

      while delta_bc > 255:
        lnotab.extend([255, 0])
        delta_bc -= 255

      if delta_lineno < 0:
        while delta_lineno < -128:
          lnotab.extend([delta_bc, 0x80])
          delta_bc = 0
          delta_lineno += 128

        delta_lineno %= 256
      else:
        while delta_lineno > 127:
          lnotab.extend([delta_bc, 127])
          delta_bc = 0
          delta_lineno -= 127

      lnotab.extend([delta_bc, delta_lineno])
      current_lineno = listing[i].lineno

    return bytes(lnotab)



#!/usr/bin/python3

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" An example native JSON vs uJSON differential fuzzer.

This fuzzer looks for differences between the built-in json library and the
native ujson library. The ujson library should be built for coverage and,
optionally, Address Sanitizer.
(see build_install_ujson.sh and the instructions for sanitizers:
https://github.com/google/atheris/blob/master/using_sanitizers.md)

This fuzzer has found a bug with inconsistent handling of integers with
too-high magnitude. uJSON sometimes refuses to process numbers that are too far
from 0 with "Value is too big!" or the equivalent for values that are too
negative. However, other times it happily processes them with two's compliment
mod. As an example, it refuses to parse "-9223372036854775809" (the first
integer not representable in a 64-bit signed number) with "Value is too small";
but it will happily parse "-80888888888888888888", a significantly more negative
number. However, it parses it as -9223372036854775808. The JSON spec
(https://tools.ietf.org/html/rfc7159#section-6) "allows implementations to set
limits on the range and precision of numbers accepted", so failing to parse
values that are too big or too small is techincally fine; however,
misinterpreting them is not.
"""

import atheris
import sys
with atheris.instrument_imports():
  import json
  import ujson


@atheris.instrument_func
def ClearAllIntegers(data):
  """Used to prevent known bug; sets all integers in data recursively to 0."""
  if type(data) == int:
    return 0
  if type(data) == list:
    for i in range(0, len(data)):
      data[i] = ClearAllIntegers(data[i])
  if type(data) == dict:
    for k, v in data:
      data[k] = ClearAllIntegers(v)
  return data


@atheris.instrument_func
def TestOneInput(input_bytes):
  fdp = atheris.FuzzedDataProvider(input_bytes)
  original = fdp.ConsumeUnicode(sys.maxsize)

  try:
    ujson_data = ujson.loads(original)
    json_data = json.loads(original)
  except Exception as e:
    # It would be interesting to enforce that if one of the libraries throws an
    # exception, the other does too. However, uJSON accepts many invalid inputs
    # that are uninteresting, such as "00". So, that is not done.
    return

  # Uncomment these lines to ignore the errors described in the docstring of
  # this file.
  json_data = ClearAllIntegers(json_data)
  ujson_data = ClearAllIntegers(ujson_data)

  json_dumped = json.dumps(json_data)
  ujson_dumped = json.dumps(ujson_data)

  if json_dumped != ujson_dumped:
    raise RuntimeError(
        "Decoding/encoding disagreement!\nInput: %s\nJSON data: %s\nuJSON data: %s\nJSON-dumped: %s\nuJSON-dumped: %s\n"
        % (original, json_data, ujson_data, json_dumped, ujson_dumped))


def main():
  atheris.Setup(sys.argv, TestOneInput)
  atheris.Fuzz()


if __name__ == "__main__":
  main()



"""Do not use this. It's a compatibility wrapper for old fuzzers."""

import sys

sys.stderr.write("WARNING: atheris_no_libfuzzer is no longer needed. "
                 "You can now just `import atheris`.\n")
del sys

from atheris import *



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A helper library for coverage_test.py - coverage is added to this library."""


def simple_func(a):
  return 2 * a


def if_func(a):
  x = a
  if x:
    return 2
  else:
    return 3


def cmp_less(a, b):
  return a < b


def cmp_greater(a, b):
  return a > b


def cmp_const_less(a):
  return 1 < a


def cmp_const_less_inverted(a):
  return a < 1


def regex_match(re_obj, a):
  re_obj.match(a)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import unittest
import zlib

import atheris

import fuzz_test_lib


def noop_mutator(data, max_size, seed):
  print("Hello from mutator")
  res = atheris.Mutate(data, len(data))
  return res


def noop_crossover(data1, data2, max_size, seed):
  print("Hello from crossover")
  return data1 + data2


@atheris.instrument_func
def test_one_input(data):
  if data == b"AA":
    raise ("Solved!")


class CustomMutatorAndCrossoverTests(unittest.TestCase):

  def testMutator(self):
    fuzz_test_lib.run_fuzztest(
        test_one_input,
        setup_kwargs={
            "custom_mutator": noop_mutator,
            "custom_crossover": noop_crossover
        },
        expected_output=b"Hello from mutator")

  def testCrossover(self):
    fuzz_test_lib.run_fuzztest(
        test_one_input,
        setup_kwargs={
            "custom_mutator": noop_mutator,
            "custom_crossover": noop_crossover
        },
        expected_output=b"Hello from crossover")


if __name__ == "__main__":
  unittest.main()



#!/usr/bin/python3
# coding=utf-8

# Copyright 2020 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""IDNA encoding/decoding differential fuzzer for Python idna vs libidn2.

This is a differential fuzzer that compares the Python `idna` package with the C
`libidn2` package. Unlike idna_uts46_fuzzer.py, it looks primarily for whether
a domain is valid or not.

To run this fuzzer, you'll need to install a thin wrapper to make libidn2
callable from Python; install libidn2, then cd to `libidn2_wrapper/` and run
`pip3 install .`.

This has found several situations where valid domains are rejected by libidn2,
including :
 -
 `髦暩晦晦晦獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳筳獳싂.퐀쓄쓄쓄쓄쓄쓄쓄쓄쓄쓄쓄쓼쓄쓄쓄쓄쓄쓄쓄쓄쓄㻄쓄쓄럄䄀싂.뼀猀獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳獳ⱁ㩁`
   libidn2 reports "domain too long", but it is not.
 - `ਗ਼.ÿ߽̃̃̃` (hex codepoints: ['a17', 'a3c', '2e', 'ff', '7fd', '303', '303',
 '303'])
   libidn2 reports "forbidden bidirectional properties", but the string is
   valid. This is likely because libidn2 uses Unicode 11, but relies on
   libunistring, which depends on Unicode 9. Therefore, it doesn't have correct
   bidirectional metadata some codepoints.
"""

import atheris
import sys
import idna
import libidn2
import unicodedata


def ShouldFail(domain):
  """Returns True for domains that we know are invalid, False otherwise."""
  if "." not in domain:
    return True
  pieces = domain.split(".")

  total_length = len(b".".join([piece.encode("punycode") for piece in pieces]))
  if total_length > 253:
    return True

  for piece in pieces:
    # Iteration over each label in the domain, checking various requirements.
    if len(piece) == 0:
      return True
    if len(piece) > 63:
      return True
    if len(piece.encode("punycode")) > 59:
      return True
    # Domain labels must not start with a -, end with a -, or have both their
    # third and fourth characters be --.
    if piece.startswith("-"):
      return True
    if piece.endswith("-"):
      return True
    if len(piece) >= 4 and piece[2] == "-" and piece[3] == "-":
      return True
    if len(piece) and unicodedata.category(piece[0])[0] == "M":
      return True

    # Bidirectional checks (ensures that the label follows the "bidi rule"
    # for IDNA)
    direction = unicodedata.bidirectional(piece[0])
    if direction in ["R", "AL"]:
      rtl = True
    elif direction == "L":
      rtl = False
    else:
      return True
    if rtl:
      has_en = False
      has_an = False
      for c in piece:
        biditype = unicodedata.bidirectional(c)
        if biditype not in [
            "R", "AL", "AN", "EN", "ES", "CS", "ET", "ON", "BN", "NSM"
        ]:
          return True
        if biditype == "EN":
          has_en = True
        if biditype == "AN":
          has_an = True
      if has_en and has_an:
        return True
      for i in range(len(piece) - 1, 0 - 1, -1):
        biditype = unicodedata.bidirectional(piece[i])
        if biditype in ["R", "AL", "EN", "AN"]:
          break
        if biditype != "NSM":
          return True

    else:
      for c in piece:
        if unicodedata.bidirectional(c) not in [
            "L", "EN", "ES", "CS", "ET", "ON", "BN", "NSM"
        ]:
          return True
      for i in range(len(piece) - 1, 0 - 1, -1):
        biditype = unicodedata.bidirectional(piece[i])
        if biditype in ["L", "EN"]:
          break
        if biditype != "NSM":
          return True
  return False


def CompareEncodedWithLibidn2(original, encoded):
  """Encodes `original` with libidn2 and compares it to `encoded`."""
  try:
    libidn2_encoded = libidn2.encode(original)
  except RuntimeError as e:
    # libidn2 only supports Unicode 11, which might mean it doesn't accept a
    # character that Python accepts. That's fine.
    if str(e).startswith("RuntimeError: string contains a disallowed "):
      return

    codepoints = [hex(ord(x))[2:] for x in original]
    sys.stderr.write(
        ("IDNA produced a valid output, whereas libidn2 returned an error.\n"
         "Input: %s\nInput codepoints: %s\nIDNA encoding: %s\n"
         "libidn2 decoding of IDNA encoding:%s\n") %
        (original, codepoints, encoded, libidn2.decode(encoded)))
    raise

  if encoded != libidn2_encoded:
    raise RuntimeError(
        ("IDNA encoding disagrees with libidn2 encoding.\nInput: %s\n"
         "IDNA encoding: %s\nlibidn2 encoding: %s") %
        (original, encoded, libidn2_encoded))


def CompareDecodedWithLibidn2(original, encoded, decoded):
  """Decodes `encoded` with libidn2, and compares it to `decoded` from idna."""
  libidn2_decoded = libidn2.decode(encoded)
  if libidn2_decoded != decoded:
    raise RuntimeError(
        ("IDNA decoding disagrees with libidn2 decoding.\nOriginal Input: %s\n"
         "Encoding: %s\nIDNA Decoding: %s\nlibidn2 Decoding: %s\n") %
        (original, encoded, decoded, libidn2_decoded))


def TestOneInput(input_bytes):
  fdp = atheris.FuzzedDataProvider(input_bytes)
  original = fdp.ConsumeUnicode(253)
  original = unicodedata.normalize("NFC", original)

  should_fail = ShouldFail(original)

  try:
    encoded = idna.encode(original, strict=True).lower()

    if should_fail:
      raise RuntimeError(
          ("Input '%s' is invalid, should have failed; "
           "however, actually encoded to '%s'") % (original, encoded))
  # These errors are very complex would essentially require the idna package to
  # be reimplemented in order to correctly implement, so we assume they are
  # valid.
  except idna.core.InvalidCodepoint as e:
    return
  except idna.core.InvalidCodepointContext as e:
    return

  except idna.core.IDNAError as e:
    if str(e).startswith("Unknown codepoint adjacent to"):
      return

    if should_fail:
      return
    codepoints = [ord(x) for x in original.lower()]
    sys.stderr.write("Input: %s\nCodepoints: %s\n" % (original, codepoints))
    raise
  except BaseException as e:
    if should_fail:
      return
    codepoints = [ord(x) for x in original.lower()]
    sys.stderr.write("Input: %s\nCodepoints: %s\n" % (original, codepoints))
    raise
  decoded = idna.decode(encoded)

  CompareEncodedWithLibidn2(original, encoded)
  CompareDecodedWithLibidn2(original, encoded, decoded)


def main():
  atheris.instrument_all()
  atheris.Setup(sys.argv, TestOneInput)
  atheris.Fuzz()


if __name__ == "__main__":
  main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fcntl
import os
import sys
import time

import atheris


def _set_nonblocking(fd):
  """Set the specified fd to a nonblocking mode."""
  oflags = fcntl.fcntl(fd, fcntl.F_GETFL)
  nflags = oflags | os.O_NONBLOCK
  fcntl.fcntl(fd, fcntl.F_SETFL, nflags)


def _fuzztest_child(test_one_input, custom_setup, setup_kwargs, pipe, args,
                    enabled_hooks):
  """Fuzzing target to run as a separate process."""
  os.close(pipe[0])
  os.dup2(pipe[1], 1)
  os.dup2(pipe[1], 2)

  if not setup_kwargs:
    setup_kwargs = {}

  try:
    if enabled_hooks:
      for hook in enabled_hooks:
        atheris.enabled_hooks.add(hook)
    custom_setup(
        [sys.argv[0]] + (args if args else []),
        test_one_input,
        **setup_kwargs)
    atheris.Fuzz()

    # To avoid running tests multiple times due to fork(), never allow control
    # flow to proceed past here. Report that we're exiting gracefully so that
    # tests can verify that's what happened.
  except SystemExit as e:
    print("Exiting gracefully.")
    sys.stdout.flush()
    os._exit(e.code)
  finally:
    print("Exiting gracefully.")
    sys.stdout.flush()
    os._exit(0)


def run_fuzztest(test_one_input,
                 custom_setup=None,
                 setup_kwargs=None,
                 expected_output=None,
                 timeout=10,
                 args=None,
                 enabled_hooks=None):
  """Fuzz test_one_input() in a subprocess.

  This forks a child, and in the child, runs atheris.Setup(test_one_input) and
  atheris.Fuzz(). Expects the fuzzer to quickly find a crash.

  Args:
    test_one_input: a callable that takes a bytes.
    custom_setup: a custom setup function, if None atheris.Setup is used.
    setup_kwargs: arguments to pass to the setup function.
    expected_output: bytes. If specified, the output of the fuzzer must contain
      this data.
    timeout: float. Time until the fuzzing is aborted and an assertion failure
      is raised.
    args: additional command-line arguments to pass to the fuzzing run.
    enabled_hooks: list of hooks.

  Raises:
    TimeoutError: when the target fuzz did not complete in the expected time.
  """
  pipe = os.pipe()

  pid = os.fork()
  if pid == 0:
    _fuzztest_child(test_one_input, custom_setup or atheris.Setup,
                    setup_kwargs, pipe, args, enabled_hooks)

  os.close(pipe[1])
  _set_nonblocking(pipe[0])

  stdout = b""
  start_time = time.time()
  while True:
    data = b""
    try:
      data = os.read(pipe[0], 1024)
    except BlockingIOError:
      pass

    sys.stderr.buffer.write(data)
    stdout += data

    if data:
      continue

    wpid = os.waitpid(pid, os.WNOHANG)

    if wpid == (0, 0):
      # Process not done yet
      if time.time() > start_time + timeout:
        raise TimeoutError("Fuzz target failed to exit within expected time.")
      time.sleep(0.1)
      continue

    # Process done, get any remaining output.
    with os.fdopen(pipe[0], "rb") as f:
      data = f.read()
    sys.stderr.buffer.write(data)
    stdout += data
    break

  if expected_output:
    if expected_output not in stdout:
      raise AssertionError("Fuzz target did not produce the expected output "
                           f"{expected_output}; actually got:\n{stdout}")



# Copyright 2020 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Setuptools for Atheris."""

import os
import shutil
import subprocess
import sys
import tempfile

import setuptools
from setuptools import Extension
from setuptools import setup
from setuptools.command.build_ext import build_ext

__version__ = os.getenv("ATHERIS_VERSION", "2.0.11")

if len(sys.argv) > 1 and sys.argv[1] == "print_version":
  print(__version__)
  quit()

clang_install_instructions = """download and build the latest version of Clang:
    git clone https://github.com/llvm/llvm-project.git
    cd llvm-project
    mkdir build
    cd build
    cmake -DLLVM_ENABLE_PROJECTS='clang;compiler-rt' -G "Unix Makefiles" ../llvm
    make -j 100  # This step is very slow
Then, set $CLANG_BIN="$(pwd)/bin/clang" and run pip again.
You should use this same Clang for building any Python extensions you plan to fuzz.
"""

too_old_error = """Your libFuzzer version is too old; set either $CLANG_BIN to point to a more recent Clang, or $LIBFUZZER_VERSION to point directly to a more recent libFuzzer .a file. If needed, """ + clang_install_instructions

no_libfuzzer_error = """Failed to find libFuzzer; set either $CLANG_BIN to point to your Clang binary, or $LIBFUZZER_LIB to point directly to your libFuzzer .a file. If needed, """ + clang_install_instructions

if sys.platform == "darwin":
  too_old_error = ("Your libFuzzer version is too old.\nPlease" +
                   clang_install_instructions + "Do not use Apple "
                   "Clang; Apple Clang does not come with libFuzzer.")
  no_libfuzzer_error = ("Failed to find libFuzzer; you may be building using "
                        "Apple Clang. Apple Clang does not come with "
                        "libFuzzer.\nPlease " + clang_install_instructions)


class PybindIncludeGetter(object):
  """Helper class to determine the pybind11 include path.

    The purpose of this class is to postpone importing pybind11
    until it is actually installed, so that the ``get_include()``
    method can be invoked.
  """

  def __str__(self):
    import pybind11  # pylint: disable=g-import-not-at-top
    return pybind11.get_include()


def check_libfuzzer_version(libfuzzer):
  """Verifies that the specified libFuzzer is of a sufficiently high version."""
  current_path = os.path.dirname(os.path.realpath(__file__))
  try:
    version = subprocess.check_output(
        [current_path + "/setup_utils/check_libfuzzer_version.sh", libfuzzer])
  except subprocess.CalledProcessError as e:
    sys.stderr.write("Failed to check libFuzzer version: %s" % e.stderr)
    sys.stderr.write("Assuming libFuxzzer is up-to-date.")
    return "up-to-date"
  version = version.strip().decode("utf-8")
  return version


def upgrade_libfuzzer(libfuzzer):
  """Hacky code for upgrading libFuzzer to be compatible with Atheris."""
  current_path = os.path.dirname(os.path.realpath(__file__))
  try:
    new_libfuzzer = subprocess.check_output(
        [current_path + "/setup_utils/upgrade_libfuzzer.sh", libfuzzer])
  except subprocess.CalledProcessError as e:
    sys.stderr.write("libFuzzer upgrade failed: %s" % e.stderr)
    return libfuzzer
  new_libfuzzer = new_libfuzzer.strip().decode("utf-8")
  return new_libfuzzer


def get_libfuzzer_lib():
  """Returns path to the libFuzzer .a library."""
  libfuzzer_lib = os.getenv("LIBFUZZER_LIB", "")
  if libfuzzer_lib:
    return libfuzzer_lib
  current_path = os.path.dirname(os.path.realpath(__file__))
  try:
    libfuzzer = subprocess.check_output(
        [current_path + "/setup_utils/find_libfuzzer.sh"])
  except subprocess.CalledProcessError as e:
    sys.stderr.write(no_libfuzzer_error + "\n")
    raise RuntimeError(no_libfuzzer_error)
  libfuzzer = libfuzzer.strip().decode("utf-8")
  return libfuzzer


ext_modules = [
    Extension(
        "atheris.native",
        sorted([
            "src/native/atheris.cc",
            "src/native/util.cc",
            "src/native/fuzzed_data_provider.cc",
        ]),
        include_dirs=[
            # Path to pybind11 headers
            PybindIncludeGetter(),
        ],
        language="c++"),
    Extension(
        "atheris.core_with_libfuzzer",
        sorted([
            "src/native/core.cc",
            "src/native/counters.cc",
            "src/native/tracer.cc",
            "src/native/util.cc",
            "src/native/timeout.cc",
        ]),
        include_dirs=[
            # Path to pybind11 headers
            PybindIncludeGetter(),
        ],
        language="c++"),
    Extension(
        "atheris.core_without_libfuzzer",
        sorted([
            "src/native/core.cc",
            "src/native/counters.cc",
            "src/native/tracer.cc",
            "src/native/util.cc",
            "src/native/timeout.cc",
        ]),
        include_dirs=[
            # Path to pybind11 headers
            PybindIncludeGetter(),
        ],
        language="c++"),
    Extension(
        "atheris.custom_crossover",
        sorted([
            "src/native/custom_crossover.cc",
            "src/native/custom_crossover_module.cc",
        ]),
        include_dirs=[
            # Path to pybind11 headers
            PybindIncludeGetter(),
        ],
        language="c++"),
    Extension(
        "atheris.custom_mutator",
        sorted([
            "src/native/custom_mutator.cc",
            "src/native/custom_mutator_module.cc",
        ]),
        include_dirs=[
            # Path to pybind11 headers
            PybindIncludeGetter(),
        ],
        language="c++"),
]


# cf http://bugs.python.org/issue26689
def has_flag(compiler, flagname):
  """Return a boolean indicating whether a flag name."""

  with tempfile.NamedTemporaryFile("w", suffix=".cpp", delete=False) as f:
    f.write("int main (int argc, char **argv) { return 0; }")
    fname = f.name
  try:
    compiler.compile([fname], extra_postargs=[flagname])
  except setuptools.distutils.errors.CompileError:
    return False
  finally:
    try:
      os.remove(fname)
    except OSError:
      pass
  return True


def cpp_flag(compiler):
  """Return the highest-supported -std=c++[11/14/17] compiler flag."""
  if os.getenv("FORCE_MIN_VERSION"):
    # Use for testing, to make sure Atheris supports C++11
    flags = ["-std=c++11"]
  elif os.getenv("FORCE_VERSION"):
    flags = ["-std=c++" + os.getenv("FORCE_VERSION")]
  else:
    flags = [
        #"-std=c++17",  C++17 disabled unless explicitly requested, to work
        # around https://github.com/pybind/pybind11/issues/1818
        "-std=c++14",
        "-std=c++11"
    ]

  for flag in flags:
    if has_flag(compiler, flag):
      return flag

  raise RuntimeError("Unsupported compiler -- at least C++11 support "
                     "is needed!")


class BuildExt(build_ext):
  """A custom build extension for adding compiler-specific options."""

  def build_extensions(self):
    libfuzzer = get_libfuzzer_lib()
    orig_libfuzzer = libfuzzer
    orig_libfuzzer_name = os.path.basename(libfuzzer)
    version = check_libfuzzer_version(libfuzzer)

    if sys.platform == "darwin" and version != "up-to-date":
      raise RuntimeError(too_old_error)

    if version == "outdated-unrecoverable":
      raise RuntimeError(too_old_error)

    elif version == "outdated-recoverable":
      sys.stderr.write("Your libFuzzer version is too old, but it's possible "
                       "to attempt an in-place upgrade. Trying that now.\n")
      libfuzzer = upgrade_libfuzzer(libfuzzer)
      if check_libfuzzer_version(libfuzzer) != "up-to-date":
        sys.stderr.write("Upgrade failed.")
        raise RuntimeError(too_old_error)
    elif version != "up-to-date":
      raise RuntimeError("Unexpected up-to-date status: " + version)

    sys.stderr.write("Your libFuzzer is up-to-date.\n")

    c_opts = ["-Wno-deprecated-declarations", "-Wno-attributes"]
    l_opts = []

    if sys.platform == "darwin":
      darwin_opts = ["-stdlib=libc++", "-mmacosx-version-min=10.7"]
      c_opts += darwin_opts
      l_opts += darwin_opts

    ct = self.compiler.compiler_type
    if ct == "unix":
      c_opts.append(cpp_flag(self.compiler))

    for ext in self.extensions:
      ext.define_macros = [("VERSION_INFO",
                            "'{}'".format(self.distribution.get_version())),
                           ("ATHERIS_MODULE_NAME", ext.name.split(".")[1])]
      ext.extra_compile_args = c_opts
      if ext.name == "atheris.core_with_libfuzzer":
        ext.extra_link_args = l_opts + [libfuzzer]
      else:
        ext.extra_link_args = l_opts
    build_ext.build_extensions(self)

    try:
      self.deploy_file(libfuzzer, orig_libfuzzer_name)
    except Exception as e:
      sys.stderr.write(str(e))
      sys.stderr.write("\n")

    # Deploy versions of ASan and UBSan that have been merged with libFuzzer
    asan_name = orig_libfuzzer.replace(".fuzzer_no_main-", ".asan-")
    merged_asan_name = "asan_with_fuzzer.so"
    self.merge_deploy_libfuzzer_sanitizer(
        libfuzzer, asan_name, merged_asan_name,
        "asan_preinit.cc.o asan_preinit.cpp.o")

    ubsan_name = orig_libfuzzer.replace(".fuzzer_no_main-",
                                        ".ubsan_standalone-")
    merged_ubsan_name = "ubsan_with_fuzzer.so"
    self.merge_deploy_libfuzzer_sanitizer(
        libfuzzer, ubsan_name, merged_ubsan_name,
        "ubsan_init_standalone_preinit.cc.o ubsan_init_standalone_preinit.cpp.o"
    )

    ubsanxx_name = orig_libfuzzer.replace(".fuzzer_no_main-",
                                          ".ubsan_standalone_cxx-")
    merged_ubsanxx_name = "ubsan_cxx_with_fuzzer.so"
    self.merge_deploy_libfuzzer_sanitizer(
        libfuzzer, ubsanxx_name, merged_ubsanxx_name,
        "ubsan_init_standalone_preinit.cc.o ubsan_init_standalone_preinit.cpp.o"
    )

  def deploy_file(self, name, target_filename):
    atheris = self.get_ext_fullpath("atheris")
    dest_file = os.path.join(os.path.dirname(atheris), target_filename)

    shutil.copy(name, dest_file)

  def merge_libfuzzer_sanitizer(self, libfuzzer, sanitizer, strip_preinit):
    """Generate a .so that contains both libFuzzer and a sanitizer."""
    current_path = os.path.dirname(os.path.realpath(__file__))

    new_sanitizer = subprocess.check_output([
        os.path.join(current_path, "setup_utils/merge_libfuzzer_sanitizer.sh"),
        libfuzzer, sanitizer, strip_preinit
    ])

    return new_sanitizer.strip().decode("utf-8")

  def merge_deploy_libfuzzer_sanitizer(self, libfuzzer, lib_name,
                                       merged_lib_name, preinit):
    try:
      merged_lib = self.merge_libfuzzer_sanitizer(libfuzzer, lib_name, preinit)
      self.deploy_file(merged_lib, merged_lib_name)
    except Exception as e:
      sys.stderr.write(str(e))
      sys.stderr.write("\n")


setup(
    name="atheris",
    version=__version__,
    author="Bitshift",
    author_email="atheris@google.com",
    url="https://github.com/google/atheris/",
    description="A coverage-guided fuzzer for Python and Python extensions.",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    packages=["atheris"],
    package_dir={"atheris": "src"},
    py_modules=["atheris_no_libfuzzer"],
    ext_modules=ext_modules,
    setup_requires=["pybind11>=2.5.0"],
    cmdclass={"build_ext": BuildExt},
    zip_safe=False,
)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides Atheris instrumentation hooks for particular functions like regex."""

import re
import sre_parse
import sys
from typing import Set, Any, Pattern, List, Match, Optional, Iterator

# mypy does not like the implicit rexport of the constants available in
# sre_parse, and also does not support ignoring for blocks of code. Rather
# than having a whole-file ignore, or interrupting every line of every statement
# below with an ignore, we will make aliases and ignore here.
_ASSERT = sre_parse.ASSERT  # type: ignore[attr-defined]
_ASSERT_NOT = sre_parse.ASSERT_NOT  # type: ignore[attr-defined]
_BRANCH = sre_parse.BRANCH  # type: ignore[attr-defined]
_CATEGORY = sre_parse.CATEGORY  # type: ignore[attr-defined]
_IN = sre_parse.IN  # type: ignore[attr-defined]
_LITERAL = sre_parse.LITERAL  # type: ignore[attr-defined]
_MAX_REPEAT = sre_parse.MAX_REPEAT  # type: ignore[attr-defined]
_NEGATE = sre_parse.NEGATE  # type: ignore[attr-defined]
_SUBPATTERN = sre_parse.SUBPATTERN  # type: ignore[attr-defined]


def gen_match(ops: Any) -> str:
  """Returns a matching string given a regex expression."""
  # TODO(cffsmith): This generator is *not* feature complete.

  available_characters = set([chr(x) for x in range(0x20, 0x7e)] + ["\t", "\n"])

  literals = ""

  for tup in ops:
    if tup[0] == _LITERAL:
      if tup[1] > 128:
        sys.stderr.write("Encountered non-ASCII char\n")
      literals += chr(tup[1])  # pytype: disable=wrong-arg-types

    elif tup[0] == _BRANCH:
      # just generate the first branch
      literals += gen_match(tup[1][1][0])

    elif tup[0] == _NEGATE:
      sys.stderr.write("WARNING: We did not expect a NEGATE op here; is " +
                       "there an invalid RegEx somewhere?\n")
      pass

    elif tup[0] == _IN:
      # Check if this class is negated.
      negated = tup[1][0][0] == _NEGATE
      # Take the first one that is actually in the class
      if not negated:
        literals += gen_match([tup[1][0]])
      else:
        char_set = set()
        # grab all literals from this class
        for t in tup[1][1:]:
          if t[0] != _LITERAL:
            sys.stderr.write("WARNING: Encountered non literal in character " +
                             "class, cannot instrument RegEx!\n")
            continue
          char_set.add(chr(t[1]))
        allowed = available_characters - char_set
        if not allowed:
          sys.stderr.write("WARNING: This character set does not seem to " +
                           "allow any characters, cannot instrument RegEx!\n")
        else:
          literals += list(allowed)[0]

    elif tup[0] == _SUBPATTERN:
      literals += gen_match(tup[1][3])

    elif tup[0] == _MAX_REPEAT:
      # The minimum amount of repetitions we need to fulfill the pattern
      minimum = tup[1][0]
      literals += gen_match(tup[1][2]) * minimum

    elif tup[0] == _ASSERT or tup[0] == _ASSERT_NOT:
      literals += gen_match(tup[1][1])

    elif tup[0] == _CATEGORY:
      sys.stderr.write("WARNING: Currently not handling RegEx categories, " +
                       "cannot instrument RegEx!\n")

    else:
      sys.stderr.write(f"WARNING: Encountered non-handled RegEx op: {tup[0]}" +
                       ", cannot instrument RegEx\n")

  return literals


def hook_re_module() -> None:
  """Adds Atheris instrumentation hooks to the `re` module."""
  pattern_gen_map = {}

  original_compile_func = re._compile  # type: ignore[attr-defined]

  def _compile_hook(pattern: str, flags: int) -> "AtherisPatternProxy":
    """Overrides re._compile."""

    generated = ""
    if pattern not in pattern_gen_map:
      pat = sre_parse.parse(pattern)
      generated = gen_match(pat)
      # Check that the pattern actually matches
      check_pattern = pattern
      try:
        # Convert our pattern to a string if necessary
        check_pattern = pattern.decode("utf-8")  # type: ignore
      except AttributeError:
        # Already a string
        pass
      except Exception as e:  # pylint: disable=broad-except
        # Not sure what went wrong.
        sys.stderr.write(f"Could not convert the pattern {pattern} to a " +
                         f"utf-8 string: {e}\n")
      try:
        if original_compile_func(check_pattern,
                                 flags).search(generated) is None:
          sys.stderr.write(f"ERROR: generated match '{generated}' did not " +
                           "match the RegEx pattern '{_pattern}'!\n")
      except Exception as e:  # pylint: disable=broad-except
        sys.stderr.write("Could not check the generated match against the " +
                         f"RegEx pattern: {e}\n")
      pattern_gen_map[pattern] = generated
    else:
      generated = pattern_gen_map[pattern]

    # Create the `re.Pattern` object. We will wrap this in a proxy later on.
    re_object = original_compile_func(pattern, flags)

    # Return the wrapped `re.Pattern` object.
    return AtherisPatternProxy(re_object, generated)

  # actually hook the `_compile` function now
  # pylint: disable=protected-access
  re._compile = _compile_hook  # type: ignore[attr-defined]
  # pylint: enable=protected-access

class EnabledHooks:
  """Manages the set of enabled hooks."""

  def __init__(self) -> None:
    self._enabled_hooks: Set[str] = set()

  def add(self, hook: str) -> None:
    hook = hook.lower()
    if hook not in list(self._enabled_hooks):
      if hook == "regex":
        hook_re_module()
        self._enabled_hooks.add(hook)


enabled_hooks = EnabledHooks()


class AtherisPatternProxy:
  """Proxy routing regex functions though Atheris tracing equivalents.

  This is a simple proxy where we can hook into various regex
  functions. This ensures that the tracing happens on each call to
  `match`, `search`, etc.

  This can be observable by users who call `compile` and then check
  if the object is actually a `re.Pattern` object.

  Unfortunately it is not possible to change the functions on the
  `re.Pattern` object itself as the functions are not writable.
  (One could try to bypass this but it would need unsafe usage from
  ctypes and probably won't be version agnostic)
  """

  # Importing at the top will not work. TODO(b/207008147): Why does it fail?
  # pylint: disable=g-import-not-at-top

  def __init__(self, re_obj: Pattern, generated: str) -> None:
    self.re_obj = re_obj
    self.generated = generated

  def search(self, string: str) -> Optional[Match[Any]]:
    from atheris import _trace_regex_match  # type: ignore[import]
    _trace_regex_match(self.generated, self.re_obj)
    return self.re_obj.search(string)

  def match(self, string: str) -> Optional[Match[Any]]:
    from atheris import _trace_regex_match  # type: ignore[import]
    _trace_regex_match(self.generated, self.re_obj)
    return self.re_obj.match(string)

  def fullmatch(self, string: str) -> Optional[Match[str]]:
    from atheris import _trace_regex_match  # type: ignore[import]
    _trace_regex_match(self.generated, self.re_obj)
    return self.re_obj.fullmatch(string)

  def findall(self, string: str) -> List[str]:
    from atheris import _trace_regex_match  # type: ignore[import]
    _trace_regex_match(self.generated, self.re_obj)
    return self.re_obj.findall(string)

  def finditer(self, string: str) -> Iterator[Match[str]]:
    from atheris import _trace_regex_match  # type: ignore[import]
    _trace_regex_match(self.generated, self.re_obj)
    return self.re_obj.finditer(string)

  def __getattr__(self, attr: str) -> Any:
    return getattr(self.re_obj, attr)

  # pylint: enable=g-import-not-at-top



# Copyright 2021 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module provides the instrumentation functionality for atheris.

Mainly the function patch_code(), which can instrument a code object and the
helper class Instrumentor.
"""
import collections
import dis
import gc
import sys
import types
from typing import Any, Callable, Iterator, List, Optional, Tuple, TypeVar, Union

from . import utils
from .native import _reserve_counter  # type: ignore[import]
from .version_dependent import add_bytes_to_jump_arg
from .version_dependent import CONDITIONAL_JUMPS
from .version_dependent import ENDS_FUNCTION
from .version_dependent import get_code_object
from .version_dependent import get_lnotab
from .version_dependent import HAVE_ABS_REFERENCE
from .version_dependent import HAVE_REL_REFERENCE
from .version_dependent import jump_arg_bytes
from .version_dependent import REVERSE_CMP_OP
from .version_dependent import UNCONDITIONAL_JUMPS

_TARGET_MODULE = "atheris"
_COVERAGE_FUNCTION = "_trace_branch"
_COMPARE_FUNCTION = "_trace_cmp"

# TODO(b/207008147): Use NewType to differentiate the many int and str types.


class Instruction:
  """A single bytecode instruction after every EXTENDED_ARG has been resolved.

  It is assumed that all instructions are always 2*n bytes long.

  Sometimes the Python-Interpreter pads instructions with 'EXTENDED_ARG 0'
  so instructions must have a minimum size.

  Attributes:
    lineno:
      Line number in the original source code.
    offset:
      Offset of an instruction in bytes.
    opcode:
      Integer identifier of the bytecode operation.
    mnemonic:
      Human readable name of the opcode.
    arg:
      Optional (default 0) argument to the instruction. This may index into
      CodeType.co_consts or it may be the address for jump instructions.
    reference:
      For jump instructions, the absolute address in bytes of the target. For
      other instructions, None.
  """

  @classmethod
  def get_fixed_size(cls) -> int:
    return 2

  def __init__(self,
               lineno: int,
               offset: int,
               opcode: int,
               arg: int = 0,
               min_size: int = 0):
    self.lineno = lineno
    self.offset = offset
    self.opcode = opcode
    self.mnemonic = dis.opname[opcode]
    self.arg = arg
    self._min_size = min_size

    if self.mnemonic in HAVE_REL_REFERENCE:
      self._is_relative: Optional[bool] = True
      self.reference: Optional[int] = self.offset + self.get_size(
      ) + jump_arg_bytes(self.arg)
    elif self.mnemonic in HAVE_ABS_REFERENCE:
      self._is_relative = False
      self.reference = jump_arg_bytes(self.arg)
    else:
      self._is_relative = None
      self.reference = None

    self.check_state()

  def __repr__(self) -> str:
    return (f"{self.mnemonic}(arg={self.arg} offset={self.offset} " +
            f"reference={self.reference} getsize={self.get_size()})")

  def has_argument(self) -> bool:
    return self.opcode >= dis.HAVE_ARGUMENT

  def _get_arg_size(self) -> int:
    if self.arg >= (1 << 24):
      return 8
    elif self.arg >= (1 << 16):
      return 6
    elif self.arg >= (1 << 8):
      return 4
    else:
      return 2

  def get_size(self) -> int:
    return max(self._get_arg_size(), self._min_size)

  def get_stack_effect(self) -> int:
    # dis.stack_effect does not work for EXTENDED_ARG and NOP
    if self.mnemonic in ["EXTENDED_ARG", "NOP"]:
      return 0

    return dis.stack_effect(self.opcode,
                            (self.arg if self.has_argument() else None))

  def to_bytes(self) -> bytes:
    """Returns this instruction as bytes."""
    size = self._get_arg_size()
    arg = self.arg
    ret = [self.opcode, arg & 0xff]

    for _ in range(size // 2 - 1):
      arg >>= 8
      ret = [dis.opmap["EXTENDED_ARG"], arg & 0xff] + ret

    while len(ret) < self._min_size:
      ret = [dis.opmap["EXTENDED_ARG"], 0] + ret

    assert len(ret) == self.get_size()

    return bytes(ret)

  def adjust(self, changed_offset: int, size: int, keep_ref: bool) -> None:
    """Compensates the offsets in this instruction for a resize elsewhere.

    Relative offsets may be invalidated due to two main events:
        (1) Insertion of instructions
        (2) Change of size of a single, already existing instruction

    (1) Some instructions of size `size` (in bytes) have been inserted at offset
        `changed_offset` in the instruction listing.

    (2) An instruction at offset changed_offset` - 0.5 has increased in size.
        If `changed_offset` is self.offset + 0.5, then self has increased.

    Either way, adjust the current offset, reference and argument
    accordingly.

    TODO(b/207008147): Replace the pattern of using +0.5 as a sentinal.

    Args:
      changed_offset: The offset where instructions are inserted.
      size: The number of bytes of instructions inserted.
      keep_ref: if True, adjust our reference.
    """
    old_offset = self.offset
    old_reference = self.reference

    if old_offset < changed_offset < (old_offset + 1):
      if old_reference is not None:
        if self._is_relative:
          self.reference += size  # type: ignore[operator]
        elif old_reference > old_offset:
          self.reference += size  # type: ignore[operator]
          self.arg = add_bytes_to_jump_arg(self.arg, size)

      return

    if changed_offset <= old_offset:
      self.offset += size

    if old_reference is not None and not keep_ref:
      if changed_offset <= old_reference:
        self.reference += size  # type: ignore[operator]

      if self._is_relative:
        if old_offset < changed_offset <= old_reference:
          self.arg = add_bytes_to_jump_arg(self.arg, size)
      else:
        if changed_offset <= old_reference:
          self.arg = add_bytes_to_jump_arg(self.arg, size)

  def check_state(self) -> None:
    """Asserts that internal state is consistent."""
    assert self.mnemonic != "EXTENDED_ARG"
    assert 0 <= self.arg <= 0x7fffffff
    assert 0 <= self.opcode < 256

    if self.reference is not None:
      if self._is_relative:
        assert self.offset + self.get_size() + jump_arg_bytes(
            self.arg) == self.reference
      else:
        assert jump_arg_bytes(self.arg) == self.reference

  def is_jump(self) -> bool:
    return self.mnemonic in CONDITIONAL_JUMPS or self.mnemonic in UNCONDITIONAL_JUMPS

  def make_nop(self) -> None:
    self.opcode = dis.opmap["NOP"]
    self.mnemonic = "NOP"
    self.arg = 0
    self._is_relative = None
    self.reference = None
    self.check_state()


class BasicBlock:
  """A block of bytecode instructions and the adresses it may jump to."""

  def __init__(self, instructions: List[Instruction], last_one: bool):
    self.instructions = instructions
    self.id = instructions[0].offset

    last_instr = instructions[-1]

    if last_one or last_instr.mnemonic in ENDS_FUNCTION:
      self.edges = []
    elif last_instr.mnemonic in CONDITIONAL_JUMPS:
      self.edges = list(
          {last_instr.reference, last_instr.offset + last_instr.get_size()})
    else:
      if last_instr.reference is not None:
        self.edges = [last_instr.reference]
      else:
        self.edges = [last_instr.offset + last_instr.get_size()]

  def __iter__(self) -> Iterator[Instruction]:
    return iter(self.instructions)

  def __repr__(self) -> str:
    return (f"BasicBlock(id={self.id}, edges={self.edges}, " +
            f"instructions={self.instructions})")


_SizeAndInstructions = Tuple[int, List[Instruction]]


class Instrumentor:
  """Implements the core instrumentation functionality.

  It gets a single code object, builds a CFG of the bytecode and
  can instrument the code for coverage collection via trace_control_flow()
  and for data-flow tracing via trace_data_flow().

  How to insert code:
      1. Select a target basic block
      2. Build up the new code as a list of `Instruction` objects.
          Make sure to get the offsets right.
      3. Calculate the overall size needed by your new code (in bytes)
      4. Call _adjust() with your target offset and calculated size
      5. Insert your instruction list into the instruction list of the basic
      block
      6. Call _handle_size_changes()
  Take a look at trace_control_flow() and trace_data_flow() for examples.

  Note that Instrumentor only supports insertions, not deletions.
  """

  def __init__(self, code: types.CodeType):
    self._cfg: collections.OrderedDict = collections.OrderedDict()
    self.consts = list(code.co_consts)
    self._names = list(code.co_names)
    self.num_counters = 0
    self._code = code

    self._build_cfg()
    self._check_state()

  def _build_cfg(self) -> None:
    """Builds control flow graph."""
    lineno = self._code.co_firstlineno
    arg = None
    offset = None
    length = Instruction.get_fixed_size()
    instr_list = []
    basic_block_borders = []
    did_jump = False
    jump_targets = set()

    for instruction in dis.get_instructions(self._code):
      if instruction.starts_line is not None:
        lineno = instruction.starts_line

      if instruction.opname == "EXTENDED_ARG":
        if arg is None:
          arg = 0
          offset = instruction.offset

        arg <<= 8
        arg |= instruction.arg  # type: ignore[operator]
        length += Instruction.get_fixed_size()  # type: ignore[operator]

        continue

      elif arg is not None:
        assert offset is not None
        instr_list.append(
            Instruction(
                lineno,
                offset,
                instruction.opcode,
                (arg << 8) | instruction.arg,  # type: ignore[operator]
                min_size=length))
        arg = None
        offset = None
        length = Instruction.get_fixed_size()

      else:
        instr_list.append(
            Instruction(lineno, instruction.offset, instruction.opcode,
                        instruction.arg or 0))

      if instr_list[-1].reference is not None:
        jump_targets.add(instr_list[-1].reference)

    for c, instr in enumerate(instr_list):
      if instr.offset == 0 or instr.offset in jump_targets or did_jump:
        basic_block_borders.append(c)

      if instr.is_jump():
        did_jump = True
      else:
        did_jump = False

    basic_block_borders.append(len(instr_list))

    for i in range(len(basic_block_borders) - 1):
      start_of_bb = basic_block_borders[i]
      end_of_bb = basic_block_borders[i + 1]
      bb = BasicBlock(instr_list[start_of_bb:end_of_bb],
                      i == len(basic_block_borders) - 2)
      self._cfg[bb.id] = bb

  def _check_state(self) -> None:
    """Asserts that the Instrumentor is in a valid state."""
    assert self._cfg, "Control flow graph empty."
    seen_ids = set()

    for basic_block in self._cfg.values():
      assert basic_block.instructions, "BasicBlock has no instructions."

      assert basic_block.id not in seen_ids
      seen_ids.add(basic_block.id)

      for edge in basic_block.edges:
        assert edge in self._cfg, (
            f"{basic_block} has an edge, {edge}, not in CFG {self._cfg}.")

    listing = self._get_linear_instruction_listing()
    i = 0

    assert listing[0].offset == 0

    while i < len(listing) - 1:
      assert (listing[i].offset + listing[i].get_size() == listing[i +
                                                                   1].offset)
      listing[i].check_state()
      i += 1

  def _get_name(self, name: str) -> int:
    """Returns an offset to `name` in co_names, appending if necessary."""
    try:
      return self._names.index(name)
    except ValueError:
      self._names.append(name)
      return len(self._names) - 1

  def _get_const(self, constant: Union[int, types.ModuleType]) -> int:
    """Returns the index of `constant` in self.consts, inserting if needed."""
    for i in range(len(self.consts)):
      if isinstance(self.consts[i],
                    type(constant)) and self.consts[i] == constant:
        return i

    self.consts.append(constant)
    return len(self.consts) - 1

  def _get_counter(self) -> int:
    counter = _reserve_counter()
    return self._get_const(counter)

  def _adjust(self, offset: float, size: int, *keep_refs: str) -> None:
    """Adjust for `size` bytes of instructions inserted at `offset`.

    Signal all instructions that some instructions of size `size` (in bytes)
    will be inserted at offset `offset`. Sometimes it is necessary that some
    instructions do not change their reference when a new insertion happens.

    All those Instruction-objects whose reference shall not change must be
    in `keep_refs`.

    Args:
      offset: Location that new instructions are inserted at
      size: How many bytes of new instructions are being inserted.
      *keep_refs: The Instructions whose reference shall not change.
    """
    for basic_block in self._cfg.values():
      for instr in basic_block:
        instr.adjust(offset, size, instr in keep_refs)

  def _handle_size_changes(self) -> None:
    """Fixes instructions who's size increased with the last insertion.

    After insertions have been made it could be that the argument of some
    instructions crossed certain boundaries so that more EXTENDED_ARGs are
    required to build the oparg. This function identifies all of those
    instructions whose size increased with the latest insertion and adjusts all
    other instructions to the new size.
    """
    listing = self._get_linear_instruction_listing()

    while True:
      found_invalid = False
      i = 0

      while i < len(listing) - 1:
        next_offset = listing[i].offset + listing[i].get_size()

        assert next_offset >= listing[i + 1].offset, (
            "Something weird happened with the offsets at offset " +
            f"{listing[i].offset}")

        if next_offset > listing[i + 1].offset:
          delta = next_offset - listing[i + 1].offset
          self._adjust(listing[i].offset + 0.5, delta)
          found_invalid = True

        i += 1

      if not found_invalid:
        break

  def _get_linear_instruction_listing(self) -> List[Instruction]:
    listing = []
    for basic_block in self._cfg.values():
      for instr in basic_block:
        listing.append(instr)
    return listing

  def to_code(self) -> types.CodeType:
    """Returns the instrumented code object."""
    self._check_state()
    listing = self._get_linear_instruction_listing()
    code = bytes()
    stacksize = 0

    for instr in listing:
      code += instr.to_bytes()
      stacksize = max(stacksize, stacksize + instr.get_stack_effect())

    return get_code_object(self._code, stacksize, code,
                           tuple(self.consts + ["__ATHERIS_INSTRUMENTED__"]),
                           tuple(self._names), get_lnotab(self._code, listing))

  def _generate_trace_branch_invocation(self, lineno: int,
                                        offset: int) -> _SizeAndInstructions:
    """Builds the bytecode that calls atheris._trace_branch()."""
    to_insert = []
    start_offset = offset
    const_atheris = self._get_const(sys.modules[_TARGET_MODULE])
    name_cov = self._get_name(_COVERAGE_FUNCTION)

    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_atheris))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_ATTR"], name_cov))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"],
                    self._get_counter()))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["CALL_FUNCTION"], 1))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["POP_TOP"]))
    offset += to_insert[-1].get_size()

    return offset - start_offset, to_insert

  def _generate_cmp_invocation(self, op: int, lineno: int,
                               offset: int) -> _SizeAndInstructions:
    """Builds the bytecode that calls atheris._trace_cmp().

    Only call this if the two objects being compared are non-constants.

    Args:
      op: The comparison operation
      lineno: The line number of the operation
      offset: The offset to the operation instruction

    Returns:
      The size of the instructions to insert,
      The instructions to insert
    """
    to_insert = []
    start_offset = offset
    const_atheris = self._get_const(sys.modules[_TARGET_MODULE])
    name_cmp = self._get_name(_COMPARE_FUNCTION)
    const_op = self._get_const(op)
    const_counter = self._get_counter()
    const_false = self._get_const(False)

    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_atheris))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_ATTR"], name_cmp))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["ROT_THREE"]))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_op))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_counter))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_false))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["CALL_FUNCTION"], 5))
    offset += to_insert[-1].get_size()

    return offset - start_offset, to_insert

  def _generate_const_cmp_invocation(self, op: int, lineno: int, offset: int,
                                     switch: bool) -> _SizeAndInstructions:
    """Builds the bytecode that calls atheris._trace_cmp().

    Only call this if one of the objects being compared is a constant coming
    from co_consts. If `switch` is true the constant is the second argument and
    needs to be switched with the first argument.

    Args:
      op: The comparison operation.
      lineno: The line number of the operation
      offset: The initial number of instructions.
      switch: bool whether the second arg is constant instead of the first.

    Returns:
      The number of bytes to insert, and the instructions.
    """
    to_insert = []
    start_offset = offset
    const_atheris = self._get_const(sys.modules[_TARGET_MODULE])
    name_cmp = self._get_name(_COMPARE_FUNCTION)
    const_counter = self._get_counter()
    const_true = self._get_const(True)
    const_op = None

    if switch:
      const_op = self._get_const(REVERSE_CMP_OP[op])
    else:
      const_op = self._get_const(op)

    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_atheris))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_ATTR"], name_cmp))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["ROT_THREE"]))
    offset += to_insert[-1].get_size()

    if switch:
      to_insert.append(Instruction(lineno, offset, dis.opmap["ROT_TWO"]))
      offset += to_insert[-1].get_size()

    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_op))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_counter))
    offset += to_insert[-1].get_size()
    to_insert.append(
        Instruction(lineno, offset, dis.opmap["LOAD_CONST"], const_true))
    offset += to_insert[-1].get_size()
    to_insert.append(Instruction(lineno, offset, dis.opmap["CALL_FUNCTION"], 5))
    offset += to_insert[-1].get_size()

    return offset - start_offset, to_insert

  def trace_control_flow(self) -> None:
    """Insert a call to atheris._trace_branch() branch's target block.

    The argument of _trace_branch() is an id for the branch.

    The following bytecode gets inserted:
      LOAD_CONST     atheris
      LOAD_ATTR      _trace_branch
      LOAD_CONST     <id>
      CALL_FUNCTION  1
      POP_TOP                  ; _trace_branch() returns None, remove the
      return value
    """
    already_instrumented = set()

    offset = self._cfg[0].instructions[0].offset
    total_size, to_insert = self._generate_trace_branch_invocation(
        self._cfg[0].instructions[0].lineno, offset)
    self._adjust(offset, total_size)
    self._cfg[0].instructions = to_insert + self._cfg[0].instructions

    for basic_block in self._cfg.values():
      if len(basic_block.edges) == 2:
        for edge in basic_block.edges:
          bb = self._cfg[edge]

          if bb.id not in already_instrumented:
            already_instrumented.add(bb.id)
            source_instr = []
            offset = bb.instructions[0].offset

            for source_bb in self._cfg.values():
              if bb.id in source_bb.edges and source_bb.instructions[
                  -1].reference == offset:
                source_instr.append(source_bb.instructions[-1])

            total_size, to_insert = self._generate_trace_branch_invocation(
                bb.instructions[0].lineno, offset)

            self._adjust(offset, total_size, *source_instr)

            bb.instructions = to_insert + bb.instructions

    self._handle_size_changes()

  def trace_data_flow(self) -> None:
    """Instruments bytecode for data-flow tracing.

    This works by replacing the instruction COMPARE_OP with a call to
    atheris._trace_cmp(). The arguments for _trace_cmp() are as follows:
        - obj1 and obj2: The two values to compare
        - opid: argument to COMPARE_OP
        - counter: The counter for this comparison.
        - is_const: whether obj1 is a constant in co_consts.

    To detect if any of the values being compared is a constant, all push and
    pop operations have to be analyzed. If a constant appears in a comparison it
    must always be given as obj1 to _trace_cmp().

    The bytecode that gets inserted looks like this:
      LOAD_CONST     atheris
      LOAD_ATTR      _trace_cmp
      ROT_THREE                   ; move atheris._trace_cmp below the two
      objects
      LOAD_CONST     <opid>
      LOAD_CONST     <counter index>
      LOAD_CONST     <is_const>
      CALL_FUNCTION  5
    """
    stack_size = 0
    seen_consts = []

    for basic_block in self._cfg.values():
      for c, instr in enumerate(basic_block.instructions):
        if instr.mnemonic == "LOAD_CONST":
          seen_consts.append(stack_size)
        elif instr.mnemonic == "COMPARE_OP" and instr.arg <= 5:
          # Determine the two values on the top of the stack before COMPARE_OP
          consts_on_stack = [
              c for c in seen_consts if stack_size - 2 <= c < stack_size
          ]
          tos_is_constant = stack_size - 1 in consts_on_stack
          tos1_is_constant = stack_size - 2 in consts_on_stack

          if not (tos_is_constant and tos1_is_constant):
            offset = instr.offset
            total_size = None
            to_insert = None

            # Both items are non-constants
            if (not tos_is_constant) and (not tos1_is_constant):
              total_size, to_insert = self._generate_cmp_invocation(
                  instr.arg, instr.lineno, offset)

            # One item is constant, one is non-constant
            else:
              total_size, to_insert = self._generate_const_cmp_invocation(
                  instr.arg, instr.lineno, offset, tos_is_constant)

            self._adjust(offset, total_size)

            for i, new_instr in enumerate(to_insert):
              basic_block.instructions.insert(c + i, new_instr)

            instr.make_nop()

        stack_size += instr.get_stack_effect()
        seen_consts = [c for c in seen_consts if c < stack_size]

    self._handle_size_changes()

  def _print_disassembly(self) -> None:
    """Prints disassembly."""
    print(f"Disassembly of {self._code.co_filename}:{self._code.co_name}")
    for basic_block in self._cfg.values():
      print(" -bb-")
      for instr in basic_block:
        print(f" L.{instr.lineno}  [{instr.offset}]  {instr.mnemonic} ", end="")

        if instr.has_argument():
          print(f"{instr.arg} ", end="")

          if instr._is_relative:
            print(f"(to {instr.reference})", end="")

        print()


def patch_code(code: types.CodeType,
               trace_dataflow: bool,
               nested: bool = False) -> types.CodeType:
  """Returns code, patched with Atheris instrumentation.

  Args:
    code: The byte code to instrument.
    trace_dataflow: Whether to trace dataflow or not.
    nested: If False, reserve counters, and patch modules. Recursive calls to
      this function are considered nested.
  """
  inst = Instrumentor(code)

  # If this code object has already been instrumented, skip it
  for const in inst.consts:
    # This avoids comparison between str and bytes (BytesWarning).
    if isinstance(const, str) and const == "__ATHERIS_INSTRUMENTED__":
      return code

  inst.trace_control_flow()

  if trace_dataflow:
    inst.trace_data_flow()

  # Repeat this for all nested code objects
  for i in range(len(inst.consts)):
    if isinstance(inst.consts[i], types.CodeType):
      if (inst.consts[i].co_name == "<lambda>" or
          (not nested and inst.consts[i].co_name == "<module>") or
          inst.consts[i].co_name[0] != "<" or
          inst.consts[i].co_name[-1] != ">"):
        inst.consts[i] = patch_code(inst.consts[i], trace_dataflow, nested=True)

  return inst.to_code()


T = TypeVar("T")


def instrument_func(func: Callable[..., T]) -> Callable[..., T]:
  """Add Atheris instrumentation to a specific function."""
  func.__code__ = patch_code(func.__code__, True, True)

  return func


def _is_instrumentable(obj: Any) -> bool:
  """Returns True if this object can be instrumented."""
  try:
    # Only callables can be instrumented
    if not hasattr(obj, "__call__"):
      return False
    # Only objects with a __code__ member of type CodeType can be instrumented
    if not hasattr(obj, "__code__"):
      return False
    if not isinstance(obj.__code__, types.CodeType):
      return False
    # Only code in a real module can be instrumented
    if not hasattr(obj, "__module__"):
      return False
    if obj.__module__ not in sys.modules:
      return False
    # Bound methods can't be instrumented - instrument the real func instead
    if hasattr(obj, "__self__"):
      return False
    # Only Python functions and methods can be instrumented, nothing native
    if (not isinstance(obj, types.FunctionType)) and (not isinstance(
        obj, types.MethodType)):
      return False
  except Exception:  # pylint: disable=broad-except
    # If accessing any of those fields produced an exception, the object
    # probably can't be instrumented
    return False

  return True


def instrument_all() -> None:
  """Add Atheris instrementation to all Python code already imported.

  This function is experimental.

  This function is able to instrument core library functions that can't be
  instrumented by instrument_func or instrument_imports, as those functions are
  used in the implementation of the instrumentation.
  """
  progress_renderer = None

  funcs = [obj for obj in gc.get_objects() if _is_instrumentable(obj)]
  if sys.stderr.isatty():
    sys.stderr.write("INFO: Instrumenting functions: ")
    progress_renderer = utils.ProgressRenderer(sys.stderr, len(funcs))
  else:
    sys.stderr.write(f"INFO: Instrumenting {len(funcs)} functions...\n")

  for i in range(len(funcs)):
    func = funcs[i]
    try:
      instrument_func(func)
    except Exception as e:  # pylint: disable=broad-except
      if progress_renderer:
        progress_renderer.drop()
      sys.stderr.write(f"ERROR: Failed to instrument function {func}: {e}\n")
    if progress_renderer:
      progress_renderer.count = i + 1

  if progress_renderer:
    progress_renderer.drop()
  else:
    print("INFO: Instrumentation complete.")



#!/usr/bin/python3

# Copyright 2020 Google LLC
# Copyright 2021 Fraunhofer FKIE
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example of fuzzing in Python."""

import atheris
import sys

# This tells Atheris to instrument all functions in the `struct` and
# `example_library` modules.
with atheris.instrument_imports():
  import struct
  import example_library


@atheris.instrument_func  # Instrument the TestOneInput function itself
def TestOneInput(data):
  """The entry point for our fuzzer.

  This is a callback that will be repeatedly invoked with different arguments
  after Fuzz() is called.
  We translate the arbitrary byte string into a format our function being fuzzed
  can understand, then call it.

  Args:
    data: Bytestring coming from the fuzzing engine.
  """
  if len(data) != 4:
    return  # Input must be 4 byte integer.

  number, = struct.unpack('<I', data)
  example_library.CodeBeingFuzzed(number)


atheris.Setup(sys.argv, TestOneInput)
atheris.Fuzz()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for coverage instrumentation."""

import dis
import re
import unittest
from unittest import mock

import atheris

with atheris.instrument_imports():
  import coverage_test_helper

# Enable RegEx instrumentation.
atheris.enabled_hooks.add("RegEx")


@atheris.instrument_func
def decorator_instrumented(x):
  return 2 * x


@atheris.instrument_func
@atheris.instrument_func
@atheris.instrument_func
@atheris.instrument_func
@atheris.instrument_func
def multi_instrumented(x):
  return 2 * x


original_trace_cmp = atheris._trace_cmp


@mock.patch.object(atheris, "_trace_regex_match")
@mock.patch.object(atheris, "_trace_cmp")
@mock.patch.object(atheris, "_trace_branch")
class CoverageTest(unittest.TestCase):

  def testBasicBlock(self, trace_branch_mock, trace_cmp_mock,
                     trace_regex_match_mock):
    trace_branch_mock.assert_not_called()
    coverage_test_helper.simple_func(7)
    trace_branch_mock.assert_called()

    trace_branch_mock.reset_mock()
    coverage_test_helper.simple_func(2)
    trace_branch_mock.assert_called()

  def testDecoratorBasicBlock(self, trace_branch_mock, trace_cmp_mock,
                              trace_regex_match_mock):
    trace_branch_mock.assert_not_called()
    decorator_instrumented(7)
    trace_branch_mock.assert_called()

    trace_branch_mock.reset_mock()
    decorator_instrumented(2)
    trace_branch_mock.assert_called()

  def testBranch(self, trace_branch_mock, trace_cmp_mock,
                 trace_regex_match_mock):
    trace_branch_mock.assert_not_called()
    coverage_test_helper.if_func(True)
    first_call_set = trace_branch_mock.call_args_list

    trace_branch_mock.reset_mock()
    coverage_test_helper.if_func(True)
    second_call_set = trace_branch_mock.call_args_list

    self.assertEqual(first_call_set, second_call_set)

    trace_branch_mock.reset_mock()
    coverage_test_helper.if_func(False)
    third_call_set = trace_branch_mock.call_args_list

    self.assertNotEqual(first_call_set, third_call_set)

  def testRegex(self, trace_branch_mock, trace_cmp_mock,
                trace_regex_match_mock):
    trace_branch_mock.reset_mock()
    trace_branch_mock.assert_not_called()
    trace_regex_match_mock.assert_not_called()
    coverage_test_helper.regex_match(re.compile("(Sun|Mon)day"), "Sunday")
    trace_branch_mock.assert_called()
    trace_regex_match_mock.assert_called()

  def assertTraceCmpWas(self, call_args, left, right, op, left_is_const):
    """Compare a _trace_cmp call to expected values."""
    #call_args: tuple(left, right, opid, idx, left_is_const)
    self.assertEqual(call_args[0], left)
    self.assertEqual(call_args[1], right)
    self.assertEqual(dis.cmp_op[call_args[2]], op)
    self.assertEqual(call_args[4], left_is_const)

  def testCompare(self, trace_branch_mock, trace_cmp_mock,
                  trace_regex_match_mock):
    trace_cmp_mock.side_effect = original_trace_cmp

    self.assertTrue(coverage_test_helper.cmp_less(1, 2))
    self.assertTraceCmpWas(trace_cmp_mock.call_args[0], 1, 2, "<", False)
    first_cmp_idx = trace_cmp_mock.call_args[0][3]
    trace_cmp_mock.reset_mock()

    self.assertFalse(coverage_test_helper.cmp_greater(1, 2))
    self.assertTraceCmpWas(trace_cmp_mock.call_args[0], 1, 2, ">", False)
    second_cmp_idx = trace_cmp_mock.call_args[0][3]
    self.assertNotEqual(first_cmp_idx, second_cmp_idx)
    trace_cmp_mock.reset_mock()

    self.assertTrue(coverage_test_helper.cmp_less(1, 2))
    self.assertTraceCmpWas(trace_cmp_mock.call_args[0], 1, 2, "<", False)
    third_cmp_idx = trace_cmp_mock.call_args[0][3]
    self.assertEqual(first_cmp_idx, third_cmp_idx)

  def testConstCompare(self, trace_branch_mock, trace_cmp_mock,
                       trace_regex_match_mock):
    trace_cmp_mock.side_effect = original_trace_cmp

    self.assertTrue(coverage_test_helper.cmp_const_less(2))
    self.assertTraceCmpWas(trace_cmp_mock.call_args[0], 1, 2, "<", True)
    first_cmp_idx = trace_cmp_mock.call_args[0][3]
    trace_cmp_mock.reset_mock()

    self.assertFalse(coverage_test_helper.cmp_const_less_inverted(3))
    self.assertTraceCmpWas(trace_cmp_mock.call_args[0], 1, 3, ">", True)
    first_cmp_idx = trace_cmp_mock.call_args[0][3]
    trace_cmp_mock.reset_mock()

  def testInstrumentationAppliedOnce(self, trace_branch_mock, trace_cmp_mock,
                                     trace_regex_match_mock):
    trace_branch_mock.assert_not_called()
    multi_instrumented(7)
    trace_branch_mock.assert_called_once()


if __name__ == "__main__":
  unittest.main()
